@@ -29,16 +29,15 @@
 #include "../../interfaces/idebugcontroller.h"
 #include "../../interfaces/iprojectcontroller.h"
 #include "../interfaces/isession.h"
-#include "../interfaces/istackcontroller.h"
 #include "../../shell/debugcontroller.h"
 
 namespace KDevelop {
 
-FrameStackModel::FrameStackModel(DebugController* parent)
-    : QAbstractItemModel(parent), m_autoUpdate(false), m_activeThread(0)
+FrameStackModel::FrameStackModel(IDebugSession *session)
+: IFrameStackModel(session), m_currentThread(-1), m_currentFrame(-1)
 {
-    connect(parent, SIGNAL(sessionAdded(KDevelop::IDebugSession*)),
-                SLOT(sessionAdded(KDevelop::IDebugSession*)));
+   connect(session, SIGNAL(stateChanged(KDevelop::IDebugSession::DebuggerState)),
+           SLOT(stateChanged(KDevelop::IDebugSession::DebuggerState)));
 }
 
 FrameStackModel::~FrameStackModel()
@@ -55,19 +54,6 @@ void FrameStackModel::setThreads(const QList<ThreadItem> &threads)
         endRemoveRows();
     }
 
-    bool activeThreadIsValid = false;
-    int currentThreadNr = 0;
-    foreach (const ThreadItem &i, threads) {
-        if (m_activeThread == i.nr) {
-            activeThreadIsValid = true;
-            break;
-        }
-        if (i.isCurrent) currentThreadNr = i.nr;
-    }
-    if (!activeThreadIsValid) {
-        setActiveThread(currentThreadNr);
-    }
-
     if (!threads.isEmpty()) {
         beginInsertRows(QModelIndex(), 0, threads.count()-1);
         m_threads = threads;
@@ -101,6 +87,11 @@ void FrameStackModel::setFrames(int threadNumber, QList<FrameItem> frames)
     beginInsertRows(threadIndex, 0, frames.count()-1);
     m_frames[threadNumber] = frames;
     endInsertRows();
+
+    // FIXME: Ugly hack. Apparently, when rows are added, the selection
+    // in the view is cleared. Emit this so that some frame is still
+    // selected.
+    emit currentFrameChanged(m_currentFrame);
 }
 
 void FrameStackModel::insertFrames(int threadNumber, const QList<FrameItem> &frames)
@@ -223,44 +214,44 @@ QVariant FrameStackModel::headerData(int section, Qt::Orientation orientation, i
     return QAbstractItemModel::headerData(section, orientation, role);
 }
 
-
-void FrameStackModel::sessionAdded(IDebugSession* session)
-{
-    connect(session, SIGNAL(stateChanged(KDevelop::IDebugSession::DebuggerState)),
-                SLOT(stateChanged(KDevelop::IDebugSession::DebuggerState)));
-}
-
-void FrameStackModel::setActiveThread(int threadNumber)
+void FrameStackModel::setCurrentThread(int threadNumber)
 {
     kDebug() << threadNumber;
-    if (m_activeThread != threadNumber) {
-        IDebugSession *session = ICore::self()->debugController()->currentSession();
-        if (session) {
-            session->stackController()->fetchFrames(threadNumber, 0, 20);
-        }
+    if (m_currentThread != threadNumber && threadNumber != -1) {
+        // FIXME: this logic means that if we switch to thread 3 and
+        // then to thread 2 and then to thread 3, we'll request frames
+        // for thread 3 again, even if the program was not run in between
+        // and therefore frames could not have changed.
+        fetchFrames(threadNumber, 0, 20);
     }
-    m_activeThread = threadNumber;
+    bool changed = (threadNumber != m_currentThread) || (m_currentFrame != 0);
+    m_currentThread = threadNumber;
+    m_currentFrame = 0;
+    emit currentThreadChanged(threadNumber);
+    emit currentFrameChanged(0);
+    if (changed)
+        session()->raiseEvent(IDebugSession::thread_or_frame_changed);
 }
 
-void FrameStackModel::setActiveThread(const QModelIndex& index)
+void FrameStackModel::setCurrentThread(const QModelIndex& index)
 {
     Q_ASSERT(index.isValid());
     Q_ASSERT(!index.internalId());
     Q_ASSERT(index.column() == 0);
-    setActiveThread(m_threads[index.row()].nr);
+    setCurrentThread(m_threads[index.row()].nr);
 }
 
 
-int FrameStackModel::activeThread() const
+int FrameStackModel::currentThread() const
 {
-    return m_activeThread;
+    return m_currentThread;
 }
 
-QModelIndex FrameStackModel::activeThreadIndex() const
+QModelIndex FrameStackModel::currentThreadIndex() const
 {
     int i = 0;
     foreach (const ThreadItem &t, m_threads) {
-        if (t.nr == activeThread()) {
+        if (t.nr == currentThread()) {
             return index(i, 0);
         }
         ++i;
@@ -268,50 +259,61 @@ QModelIndex FrameStackModel::activeThreadIndex() const
     return QModelIndex();
 }
 
-void FrameStackModel::setAutoUpdate(bool autoUpdate)
+int FrameStackModel::currentFrame() const
 {
-    IDebugSession *session = ICore::self()->debugController()->currentSession();
-    if (!m_autoUpdate && autoUpdate && session && (session->state() == IDebugSession::PausedState || session->state() == IDebugSession::StoppedState)) {
-        update();
+    return m_currentFrame;
+}
+
+QModelIndex FrameStackModel::currentFrameIndex() const
+{
+    QModelIndex idx = currentThreadIndex();
+    return idx.child(m_currentFrame, 0);
+}
+
+void FrameStackModel::setCurrentFrame(int frame)
+{
+    if (frame != m_currentFrame)
+    {
+        m_currentFrame = frame;
+        session()->raiseEvent(IDebugSession::thread_or_frame_changed);
+        emit currentFrameChanged(frame);
     }
-    m_autoUpdate = autoUpdate;
 }
 
 void FrameStackModel::update()
 {
-    IDebugSession *session = ICore::self()->debugController()->currentSession();
-    if (session) {
-        session->stackController()->fetchThreads();
-        if (m_activeThread) {
-            session->stackController()->fetchFrames(m_activeThread, 0, 20);
-        }
+    fetchThreads();
+    if (m_currentThread != -1) {
+        fetchFrames(m_currentThread, 0, 20);
     }
 }
 
-void FrameStackModel::stateChanged(IDebugSession::DebuggerState state)
+void FrameStackModel::handleEvent(IDebugSession::event_t event)
 {
-    kDebug() << state;
-
-    if (state == IDebugSession::EndedState && !ICore::self()->debugController()->currentSession()) {
+    switch (event)
+    {
+    case IDebugSession::debugger_exited:
+    case IDebugSession::program_exited:
         setThreads(QList<FrameStackModel::ThreadItem>());
-    }
+        break;
 
-    //ignore if not current session
-    if (sender() != ICore::self()->debugController()->currentSession()) return;
+    case IDebugSession::program_state_changed:
+        update();
+        break;
 
-    if (state == IDebugSession::PausedState || state == IDebugSession::StoppedState) {
-        if (m_autoUpdate) update();
+    default:
+        break;
     }
 }
 
-
+// FIXME: it should be possible to fetch more frames for
+// an arbitrary thread, without making it current.
 void FrameStackModel::fetchMoreFrames()
-{
-    IDebugSession *session = ICore::self()->debugController()->currentSession();
-    if (session && m_activeThread && m_hasMoreFrames[m_activeThread]) {
-        session->stackController()->fetchFrames(m_activeThread,
-                                                m_frames[m_activeThread].count(),
-                                                m_frames[m_activeThread].count()-1+20);
+{    
+    if (m_currentThread != -1 && m_hasMoreFrames[m_currentThread]) {
+        fetchFrames(m_currentThread,
+                    m_frames[m_currentThread].count(),
+                    m_frames[m_currentThread].count()-1+20);
     }
 }
 
