@@ -51,11 +51,14 @@ VariableToolTip::VariableToolTip(QWidget* parent, QPoint position,
 {
     model_ = new TreeModel(QVector<QString>() << "Name" << "Type",
                            this);
+
     TooltipRoot* tr = new TooltipRoot(model_);
     model_->setRootItem(tr);
-    tr->init(identifier);
-    var_ = tr->var;
-    var_->createVarobjMaybe(this, "variableCreated");
+    var_ = ICore::self()->debugController()->currentSession()->
+        variableController()->createVariable(
+            model_, tr, identifier);
+    tr->init(var_);
+    var_->attachMaybe(this, "variableCreated");
 
     QVBoxLayout* l = new QVBoxLayout(this);
     l->setContentsMargins(0, 0, 0, 0);
