@@ -88,13 +88,17 @@ public:
 
     virtual KUrl projectsBaseDirectory() const = 0;
 
+    enum FormattingOptions {
+        FormatHtml,
+        FormatPlain
+    };
     /**
-     * Returns a pretty short representation of the given url, considering the currently loaded projects,
-     * and with some added html highlighting:
+     * Returns a pretty short representation of the given url, considering the currently loaded projects:
      * When the file is part of a currently loaded project, that projects name is shown as prefix instead of the
      * the full file path.
+     * @param format formatting used for the string
      */
-    Q_SCRIPTABLE virtual QString prettyFileName(KUrl url) const = 0;
+    Q_SCRIPTABLE virtual QString prettyFileName(KUrl url, FormattingOptions format = FormatHtml) const = 0;
     
 public Q_SLOTS:
     /**
@@ -122,6 +126,15 @@ public Q_SLOTS:
 //     virtual void changeCurrentProject( KDevelop::ProjectBaseItem* ) = 0;
 
 Q_SIGNALS:
+    /**
+     * emitted right before a project is started to be loaded.
+     * At this point all sanity checks have been done, so the project
+     * is really going to be loaded.
+     * @note the given project won't be very useful as it doesn't have all things
+     * setup properly
+     * @param url the url of the project that is about to be opened.
+     */
+    void projectAboutToBeOpened( KDevelop::IProject* project );
     /**
      * emitted after a project is completely opened and the project model
      * has been populated.
