@@ -62,8 +62,10 @@ static QHash<KDevelop::IndexedString, QList<KDevelop::TopDUContext::Features> >
 namespace KDevelop
 {
 
-struct ParseJobPrivate
+class ParseJobPrivate
 {
+public:
+
     ParseJobPrivate(const KUrl& url) :
           document( IndexedString(url.pathOrUrl()) )
         , backgroundParser( 0 )
@@ -104,6 +106,14 @@ ParseJob::ParseJob( const KUrl &url,
 
 ParseJob::~ParseJob()
 {
+    //Here we release the revision
+    EditorIntegrator editor;
+    editor.setCurrentUrl(d->document);
+    
+    if(KDevelop::LockedSmartInterface smart = editor.smart()) {
+        smart->releaseRevision(d->revisionToken);
+        smart->clearRevision();
+    }
     {
         //Only for testing
         DUChainReadLocker lock(DUChain::lock());
@@ -180,7 +190,9 @@ bool ParseJob::contentsAvailableFromEditor()
         SmartInterface* iface = qobject_cast<SmartInterface*>(doc);
         if (iface) {
             QMutexLocker smartLock(iface->smartMutex());
-            d->revisionToken = EditorIntegrator::saveCurrentRevision(doc);
+            //Here we save the revision
+            d->revisionToken = iface->currentRevision();
+            iface->useRevision(d->revisionToken);
 
             // You must have called contentsAvailableFromEditor, it sets state
 
