@@ -2,6 +2,7 @@
     Copyright 2002-2005 Roberto Raggi <roberto@kdevelop.org>
     Copyright 2006 Adam Treat <treat@kde.org>
     Copyright 2006 Hamish Rodda <rodda@kde.org>
+    Copyright 2007-2008 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -32,7 +33,7 @@ class KDEVPLATFORMLANGUAGE_EXPORT ClassMemberDeclarationData : public Declaratio
 public:
   ClassMemberDeclarationData();
   ClassMemberDeclarationData( const ClassMemberDeclarationData& rhs );
-  
+
   Declaration::AccessPolicy m_accessPolicy;
   bool m_isStatic: 1;
   bool m_isAuto: 1;
@@ -40,6 +41,11 @@ public:
   bool m_isRegister: 1;
   bool m_isExtern: 1;
   bool m_isMutable: 1;
+  bool m_isFinal: 1;
+  bool m_isNative: 1;
+  bool m_isSynchronized: 1;
+  bool m_isStrictFP: 1;
+  bool m_isAbstract: 1;
 };
 
 }
