@@ -1,6 +1,7 @@
 /* This file is part of KDevelop
     Copyright 2007 Kris Wong <kris.p.wong@gmail.com>
     Copyright 2007 Hamish Rodda <rodda@kde.org>
+   Copyright 2007 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -61,6 +62,16 @@ namespace std {
 #endif
 }
 
+#ifndef Q_CC_MSVC
+namespace __gnu_cxx {
+    template<>
+    struct hash<void *>
+    {
+        inline size_t operator() (const void *ptr) const { return reinterpret_cast<size_t>(ptr); }
+    };
+}
+#endif
+
 namespace KDevelop
 {
 class DUChainLockPrivate
