@@ -1,5 +1,6 @@
 /* This file is part of KDevelop
     Copyright 2006 Hamish Rodda <rodda@kde.org>
+    Copyright 2007-2009 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -171,7 +172,10 @@ class KDEVPLATFORMLANGUAGE_EXPORT LocalIndexedDUContext {
 };
 
 ///This class is used to trace imports while findDeclarationsInternal. The back-tracing may be needed for correctly resolving delayed types(templates)
-struct ImportTraceItem {
+class ImportTraceItem
+{
+public:
+
   ImportTraceItem(const DUContext* _ctx, SimpleCursor _pos = SimpleCursor::invalid()) : ctx(_ctx), position(_pos) {
   }
   ImportTraceItem() {
@@ -211,6 +215,7 @@ class KDEVPLATFORMLANGUAGE_EXPORT DUContext : public DUChainBase
   friend class DUContextData;
   friend class DUContextDynamicData;
   friend class Definition;
+  friend class VisibleDeclarationIterator;
 
 public:
   /**
@@ -284,9 +289,10 @@ public:
 
   /**
    * Find the context which most specifically covers \a position.
+   * @param includeRightBorder When this is true, contexts will also be found that have the position on their right border.
    * The search is recursive, so the most specific context is found.
    */
-  DUContext* findContextAt(const SimpleCursor& position) const;
+  DUContext* findContextAt(const SimpleCursor& position, bool includeBorders = false) const;
 
   /**
    * Find a child declaration that has a rang that covers the given position
@@ -343,7 +349,11 @@ public:
 
   /// Represents an imported parent context.
   struct KDEVPLATFORMLANGUAGE_EXPORT Import {
-    Import(DUContext* context = 0, const SimpleCursor& position = SimpleCursor::invalid());
+    ///DUChain must be read-locked when this is called
+    Import(DUContext* context, const DUContext* importer, const SimpleCursor& position = SimpleCursor::invalid());
+    Import() : position(SimpleCursor::invalid()) {
+    }
+    Import(const DeclarationId& id, const SimpleCursor& position = SimpleCursor::invalid());
     bool operator==(const Import& rhs) const {
       return m_context == rhs.m_context && m_declaration == rhs.m_declaration;
     }
@@ -361,8 +371,13 @@ public:
       return m_context;
     }
 
-    ///Returns true if this import can be followed back from the imported context
-    bool isBackwardMapped() const;
+    ///returns true if this import is direct (Not referring to the import by its identifier, but rather directly by its index)
+    bool isDirect() const;
+    
+    ///If this import is indirect, returns the imported declaration-id
+    DeclarationId indirectDeclarationId() const {
+      return m_declaration;
+    }
 
     SimpleCursor position;
     private:
@@ -374,7 +389,7 @@ public:
 
   /**
    * Returns the list of imported parent contexts for this context.
-   * @warning The list may contain objects that are not valid any more(data() returns zero, but that can only happen when using anonymous imports, @see addImportedParentContext)
+   * @warning The list may contain objects that are not valid any more(data() returns zero), @see addImportedParentContext)
    * @warning The import structure may contain loops if this is a TopDUContext, so be careful when traversing the tree.
    * Expensive.
    */
@@ -405,6 +420,15 @@ public:
    */
   virtual void addImportedParentContext(DUContext* context, const SimpleCursor& position = SimpleCursor::invalid(), bool anonymous = false, bool temporary = false);
 
+  /**
+   * Adds an imported context, which may be indirect.
+   * @warning This is only allowed if this context is _NOT_ a top-context
+   * @warning When using this mechanism, this context will not be registered as importer to the other one.
+   * @warning The given import _must_ be indirect
+   * @return true if the import was already imported before, else false.
+   */
+  bool addIndirectImport(const DUContext::Import& import);
+
   /**
    * Removes a child context.
    */
@@ -431,10 +455,8 @@ public:
 
   /**
    * Cheap, because nothing needs to be loaded.
-   * Returns an array of IndexedDUContext, with the size indexedImportersSize()
    */
-  const IndexedDUContext* indexedImporters() const;
-  uint indexedImportersSize() const;
+  KDevVarLengthArray<IndexedDUContext> indexedImporters() const;
 
   /**
    * Returns the list of immediate child contexts for this context.
@@ -459,7 +481,7 @@ public:
    * \param upDistance upwards distance in the context-structure of the
    *                   given specialization-info. This allows specializing children.
    * */
-  virtual DUContext* specialize(uint specialization, const TopDUContext* topContext, int upDistance = 0);
+  virtual DUContext* specialize(IndexedInstantiationInformation specialization, const TopDUContext* topContext, int upDistance = 0);
 
   /**
    * Searches for and returns a declaration with a given \a identifier in this context, which
@@ -515,8 +537,10 @@ public:
 
   /**
    * Returns all local declarations
+   * @param source A source-context that is needed to instantiate template-declarations in some cases.
+   *               If it is zero, that signalizes that missing members should not be instantiated.
    */
-  QVector<Declaration*> localDeclarations() const;
+  virtual QVector<Declaration*> localDeclarations(const TopDUContext* source = 0) const;
 
   /**
    * Searches for the most specific context for the given cursor \a position in the given \a url.
@@ -539,6 +563,8 @@ public:
    * \returns the requested context if one was found, otherwise null.
    *
    * \warning this may return contexts which are not in this tree, you may need to lock them too...
+   * \warning this function is not as far developed as findDeclarations(..), and does not respect all of the details
+   *                so always use findDeclarations(..) instead when possible!
    */
   QList<DUContext*> findContexts(ContextType contextType, const QualifiedIdentifier& identifier, const SimpleCursor& position = SimpleCursor::invalid(), const TopDUContext* source = 0, SearchFlags flags = NoSearchFlags) const;
 
@@ -743,13 +769,13 @@ struct KDEVPLATFORMLANGUAGE_EXPORT SearchItem : public KShared {
    *   search might stop as soon as one item is found.
    * - The source top-context is needed to correctly resolve template-parameters
    * @param position A valid position, if in doubt use textRange().end()
-   *
+   * @param depth Depth of the search in parents. This is used to prevent endless recursions in endless import loops.
    * @warning position Must be valid!
    * @return whether the search was successful. If it is false, it had to be stopped for special reasons(like some flags)
    * */
   typedef KDevVarLengthArray<Declaration*, 40> DeclarationList;
 
-  virtual bool findDeclarationsInternal(const SearchItem::PtrList& identifiers, const SimpleCursor& position, const AbstractType::Ptr& dataType, DeclarationList& ret, const TopDUContext* source, SearchFlags flags ) const;
+  virtual bool findDeclarationsInternal(const SearchItem::PtrList& identifiers, const SimpleCursor& position, const AbstractType::Ptr& dataType, DeclarationList& ret, const TopDUContext* source, SearchFlags flags, uint depth ) const;
 
   ///Call this after parsing is finished. It will optimize the internal vectors to reduce memory-usage.
   void squeeze();
@@ -778,8 +804,9 @@ struct KDEVPLATFORMLANGUAGE_EXPORT SearchItem : public KShared {
 
   /**Applies namespace-imports and namespace-aliases and returns possible absolute identifiers that need to be searched.
    * @param targetIdentifiers will be filled with all identifiers that should be searched for, instead of identifier.
+   * @param onlyImports if this is true, namespace-aliases will not be respected, but only imports. This is faster.
    * */
-  void applyAliases(const SearchItem::PtrList& identifiers, SearchItem::PtrList& targetIdentifiers, const SimpleCursor& position, bool canBeNamespace) const;
+  void applyAliases(const SearchItem::PtrList& identifiers, SearchItem::PtrList& targetIdentifiers, const SimpleCursor& position, bool canBeNamespace, bool onlyImports = false) const;
   /**
    * Applies the aliases that need to be applied when moving the search from this context up to the parent-context.
    * The default-implementation adds a set of identifiers with the own local identifier prefixed, if this is a namespace.
@@ -828,7 +855,7 @@ private:
  *
  * @see NamespaceAliasDeclaration.
  * */
-KDEVPLATFORMLANGUAGE_EXPORT extern const Identifier globalImportIdentifier;
+KDEVPLATFORMLANGUAGE_EXPORT extern const Identifier& globalImportIdentifier;
 
 /**
   * Collects all uses of the given @param declarationIndex
