@@ -52,7 +52,7 @@ KDevelop::TopDUContextPointer AbstractNavigationContext::topContext() const {
 
 
 AbstractNavigationContext::AbstractNavigationContext( KDevelop::TopDUContextPointer topContext, AbstractNavigationContext* previousContext)
-  : m_shorten(false), m_selectedLink(0), m_linkCount(-1), m_currentPositionLine(0),
+  : m_selectedLink(0), m_shorten(false), m_linkCount(-1), m_currentPositionLine(0),
     m_previousContext(previousContext), m_topContext(topContext)
 {
 }
@@ -399,6 +399,11 @@ bool AbstractNavigationContext::alreadyComputed() const {
   return !m_currentText.isEmpty();
 }
 
+bool AbstractNavigationContext::isWidgetMaximized() const
+{
+  return true;
+}
+
 QWidget* AbstractNavigationContext::widget() const {
   return 0;
 }
