@@ -22,6 +22,8 @@
 
 #include <QtCore/QStringList>
 
+#include <kglobalsettings.h>
+
 namespace KDevelop
 {
 
@@ -33,14 +35,16 @@ OutputModel::OutputModel( QObject* parent )
 
 void OutputModel::appendLine( const QString& line )
 {
-    appendRow( new QStandardItem( line ) );
+    QStandardItem* item = new QStandardItem( line );
+    item->setFont( KGlobalSettings::fixedFont() );
+    appendRow( item );
 }
 
 void OutputModel::appendLines( const QStringList& lines)
 {
     Q_FOREACH( const QString& s, lines )
     {
-        appendRow( new QStandardItem( s ) );
+        appendLine( s );
     }
 }
 
