@@ -25,36 +25,10 @@
 
 #include <QtGui/QTreeView>
 
-#include "language/duchain/duchainbase.h"
-#include "language/duchain/duchainobserver.h"
 #include <language/duchain/identifier.h>
 
 class ClassBrowserPlugin;
-class KLineEdit;
 class ClassModel;
-class ClassTree;
-
-namespace KDevelop 
-{
-  class TopDUContext;
-  class IDocument;
-}
-
-/// The class browser widget
-class ClassWidget : public QWidget
-{
-  Q_OBJECT
-
-public:
-  ClassWidget(QWidget* parent, ClassBrowserPlugin* plugin);
-  virtual ~ClassWidget();
-
-private:
-  ClassBrowserPlugin* m_plugin;
-  ClassModel* m_model;
-  ClassTree* m_tree;
-  KLineEdit* m_searchLine;
-};
 
 class ClassTree : public QTreeView
 {
