@@ -59,6 +59,9 @@
 #include <language/duchain/specializationstore.h>
 #include <language/util/navigationtooltip.h>
 #include <language/duchain/navigation/abstractnavigationwidget.h>
+#include <language/interfaces/iquickopen.h>
+#include <interfaces/iplugincontroller.h>
+#include <sublime/mainwindow.h>
 
 const unsigned int highlightingTimeout = 150;
 
@@ -69,7 +72,7 @@ using KTextEditor::View;
 
 bool toolTipEnabled = true;
 
-static QWidget* masterWidget(QWidget* w) {
+QWidget* masterWidget(QWidget* w) {
   while(w && w->parent() && qobject_cast<QWidget*>(w->parent()))
     w = qobject_cast<QWidget*>(w->parent());
   return w;
@@ -80,9 +83,9 @@ class ContextBrowserViewFactory: public KDevelop::IToolViewFactory
 public:
     ContextBrowserViewFactory(ContextBrowserPlugin *plugin): m_plugin(plugin) {}
 
-    virtual QWidget* create(QWidget */*parent*/ = 0)
+    virtual QWidget* create(QWidget *parent = 0)
     {
-        ContextBrowserView* ret = new ContextBrowserView(m_plugin);
+        ContextBrowserView* ret = new ContextBrowserView(m_plugin, parent);
         QObject::connect(ret, SIGNAL(startDelayedBrowsing(KTextEditor::View*)), m_plugin, SLOT(startDelayedBrowsing(KTextEditor::View*)));
         QObject::connect(ret, SIGNAL(stopDelayedBrowsing()), m_plugin, SLOT(stopDelayedBrowsing()));
         return ret;
@@ -102,17 +105,71 @@ private:
     ContextBrowserPlugin *m_plugin;
 };
 
+void ContextBrowserPlugin::createActionsForMainWindow(Sublime::MainWindow* window, QString& xmlFile, KActionCollection& actions) {
+    xmlFile = "kdevcontextbrowser.rc" ;
+
+    KAction* previousContext = actions.addAction("previous_context");
+    previousContext->setText( i18n("&Previous Visited Context") );
+    previousContext->setIcon( KIcon("go-previous-context" ) );
+    previousContext->setShortcut( Qt::META | Qt::Key_Left );
+    QObject::connect(previousContext, SIGNAL(triggered(bool)), this, SLOT(previousContextShortcut()));
+
+    KAction* nextContext = actions.addAction("next_context");
+    nextContext->setText( i18n("&Next Visited Context") );
+    nextContext->setIcon( KIcon("go-next-context" ) );
+    nextContext->setShortcut( Qt::META | Qt::Key_Right );
+    QObject::connect(nextContext, SIGNAL(triggered(bool)), this, SLOT(nextContextShortcut()));
+
+    KAction* previousUse = actions.addAction("previous_use");
+    previousUse->setText( i18n("&Previous Use") );
+    previousUse->setIcon( KIcon("go-previous-use") );
+    previousUse->setShortcut( Qt::META | Qt::SHIFT |  Qt::Key_Left );
+    QObject::connect(previousUse, SIGNAL(triggered(bool)), this, SLOT(previousUseShortcut()));
+
+    KAction* nextUse = actions.addAction("next_use");
+    nextUse->setText( i18n("&Next Use") );
+    nextUse->setIcon( KIcon("go-next-use") );
+    nextUse->setShortcut( Qt::META | Qt::SHIFT | Qt::Key_Right );
+    QObject::connect(nextUse, SIGNAL(triggered(bool)), this, SLOT(nextUseShortcut()));
+
+    KAction* outline = actions.addAction("outline_line");
+    outline->setText(i18n("Context Browser"));
+    QWidget* w = toolbarWidgetForMainWindow(window);
+    w->setHidden(false);
+    outline->setDefaultWidget(w);
+}
+
+
+void ContextBrowserPlugin::nextContextShortcut()
+{
+  foreach(ContextBrowserView* view, m_views)
+  {
+    kDebug() << masterWidget(ICore::self()->uiController()->activeMainWindow()) << masterWidget(view);
+    if(masterWidget(ICore::self()->uiController()->activeMainWindow()) == masterWidget(view)) {
+      view->historyNext();
+      return;
+    }
+  }
+}
+
+void ContextBrowserPlugin::previousContextShortcut()
+{
+  foreach(ContextBrowserView* view, m_views)
+  {
+    if(masterWidget(ICore::self()->uiController()->activeMainWindow()) == masterWidget(view)) {
+      view->historyPrevious();
+      return;
+    }
+  }
+}
+
 K_PLUGIN_FACTORY(ContextBrowserFactory, registerPlugin<ContextBrowserPlugin>(); )
 K_EXPORT_PLUGIN(ContextBrowserFactory(KAboutData("kdevcontextbrowser","kdevcontextbrowser",ki18n("Context Browser"), "0.1", ki18n("Shows information for the current context"), KAboutData::License_GPL)))
 
 ContextBrowserPlugin::ContextBrowserPlugin(QObject *parent, const QVariantList&)
     : KDevelop::IPlugin(ContextBrowserFactory::componentData(), parent)
-    , m_viewFactory(new ContextBrowserViewFactory(this))
+    , m_backupsMutex(QMutex::Recursive), m_viewFactory(new ContextBrowserViewFactory(this))
 {
-  setXMLFile( "kdevcontextbrowser.rc" );
-
-  KActionCollection* actions = actionCollection();
-
   core()->uiController()->addToolView(i18n("Code Browser"), m_viewFactory);
 
   connect( core()->documentController(), SIGNAL( textDocumentCreated( KDevelop::IDocument* ) ), this, SLOT( textDocumentCreated( KDevelop::IDocument* ) ) );
@@ -125,27 +182,8 @@ ContextBrowserPlugin::ContextBrowserPlugin(QObject *parent, const QVariantList&)
   m_updateTimer = new QTimer(this);
   m_updateTimer->setSingleShot(true);
   connect( m_updateTimer, SIGNAL( timeout() ), this, SLOT( updateViews() ) );
-
-  KAction* previousContext = actions->addAction("previous_context");
-  previousContext->setText( i18n("&Previous Visited Context") );
-  previousContext->setShortcut( Qt::META | Qt::Key_Left );
-  connect(previousContext, SIGNAL(triggered(bool)), this, SIGNAL(previousContextShortcut()));
-
-  KAction* nextContext = actions->addAction("next_context");
-  nextContext->setText( i18n("&Next Visited Context") );
-  nextContext->setShortcut( Qt::META | Qt::Key_Right );
-  connect(nextContext, SIGNAL(triggered(bool)), this, SIGNAL(nextContextShortcut()));
-
-  KAction* previousUse = actions->addAction("previous_use");
-  previousUse->setText( i18n("&Previous Use") );
-  previousUse->setShortcut( Qt::META | Qt::SHIFT |  Qt::Key_Left );
-  connect(previousUse, SIGNAL(triggered(bool)), this, SLOT(previousUseShortcut()));
-
-  KAction* nextUse = actions->addAction("next_use");
-  nextUse->setText( i18n("&Next Use") );
-  nextUse->setShortcut( Qt::META | Qt::SHIFT | Qt::Key_Right );
-  connect(nextUse, SIGNAL(triggered(bool)), this, SLOT(nextUseShortcut()));
   
+  //Needed global action for the context-menu extensions
   m_findUses = new QAction(i18n("Find Uses"), this);
   connect(m_findUses, SIGNAL(triggered(bool)), this, SLOT(findUses()));
 }
@@ -184,6 +222,7 @@ void ContextBrowserPlugin::unload()
 }
 
 void ContextBrowserPlugin::rangeDeleted( KTextEditor::SmartRange *range ) {
+  QMutexLocker lock(&m_backupsMutex);
   m_backups.remove( range );
   m_watchedRanges.remove(range);
 
@@ -360,8 +399,8 @@ Attribute::Ptr highlightedUseAttribute(bool /*mouseHighlight*/, bool bold) {
     if( !standardBoldAttribute ) {
       standardBoldAttribute= Attribute::Ptr( new Attribute() );
       standardBoldAttribute->setBackgroundFillWhitespace(true);
-      standardBoldAttribute->setBackground(Qt::yellow);//QApplication::palette().toolTipBase());
-      standardBoldAttribute->setForeground(Qt::black); 
+      
+      standardBoldAttribute->setBackground(QColor(255, 255, 0, 150));//QApplication::palette().toolTipBase());
       standardBoldAttribute->setFontBold(true);
     }
     return standardBoldAttribute;
@@ -370,8 +409,7 @@ Attribute::Ptr highlightedUseAttribute(bool /*mouseHighlight*/, bool bold) {
     if( !standardAttribute ) {
       standardAttribute = Attribute::Ptr( new Attribute() );
       standardAttribute->setBackgroundFillWhitespace(true);
-      standardAttribute->setBackground(Qt::yellow);//QApplication::palette().toolTipBase());
-      standardAttribute->setForeground(Qt::black); 
+      standardAttribute->setBackground(QColor(255, 255, 0, 150));//QApplication::palette().toolTipBase());
     }
     return standardAttribute;
   }
@@ -392,9 +430,7 @@ Attribute::Ptr highlightedSpecialObjectAttribute() {
   if( !standardAttribute ) {
     standardAttribute = Attribute::Ptr( new Attribute() );
     standardAttribute->setBackgroundFillWhitespace(true);
-    QColor color(Qt::yellow);
-    color.setRed(90);
-    standardAttribute->setBackground(color);
+    standardAttribute->setBackground(QColor(90, 255, 0, 150));//QApplication::palette().toolTipBase());
   }
   return standardAttribute;
 }
@@ -403,6 +439,8 @@ void ContextBrowserPlugin::changeHighlight( KTextEditor::SmartRange* range, bool
   if( !range )
     return;
 
+  QMutexLocker lock(&m_backupsMutex);
+  
   if( highlight ) {
 /*    if( declaration )
       attrib = highlightedDeclarationAttribute();
@@ -460,9 +498,9 @@ bool ContextBrowserPlugin::findSpecialObject(View* view, const SimpleCursor& pos
         QMutexLocker lock(iface->smartMutex());
         //Q_ASSERT(m_highlightedRange[view]->document() == view->document());
         if (m_highlightedRange[view]->document() == view->document()) {
-	  delete m_highlightedRange[view];
+          delete m_highlightedRange[view];
           m_highlightedRange.remove(view);
-      	} else {
+      } else {
           kDebug() << "m_highlightedRange[view]->document() != view->document()";	
         }
       }
@@ -470,7 +508,7 @@ bool ContextBrowserPlugin::findSpecialObject(View* view, const SimpleCursor& pos
       KUrl viewUrl = view->document()->url();
       QList<ILanguage*> languages = ICore::self()->languageController()->languagesForUrl(viewUrl);
 
-      SimpleRange r;
+      SimpleRange r = SimpleRange::invalid();
       foreach( ILanguage* language, languages) {
         r = language->languageSupport()->specialLanguageObjectRange(viewUrl, position);
         if(r.isValid()) {
@@ -493,6 +531,7 @@ bool ContextBrowserPlugin::findSpecialObject(View* view, const SimpleCursor& pos
 
 Declaration* ContextBrowserPlugin::findDeclaration(View* view, const SimpleCursor& position, bool mouseHighlight)
 {
+      Q_UNUSED(mouseHighlight);
       Declaration* foundDeclaration = 0;
       if(m_useDeclaration.data()) {
         foundDeclaration = m_useDeclaration.data();
@@ -500,23 +539,16 @@ Declaration* ContextBrowserPlugin::findDeclaration(View* view, const SimpleCurso
         //If we haven't found a special language object, search for a use/declaration and eventually highlight it
         foundDeclaration = DUChainUtils::declarationForDefinition( DUChainUtils::itemUnderCursor(view->document()->url(), position) );
       }
-      if( foundDeclaration ) {
-        m_lastHighlightedDeclaration = IndexedDeclaration(foundDeclaration);
-        m_highlightedDeclarations[view] = foundDeclaration;
-        changeHighlight( view, foundDeclaration, true, mouseHighlight );
-      }else{
-        m_highlightedDeclarations.remove(view);
-      }
       return foundDeclaration;
 }
 
 bool ContextBrowserPlugin::showDeclarationView(View* view, const SimpleCursor& position, Declaration* foundDeclaration, DUContext* ctx)
 {
+        Q_UNUSED(position);
         Q_ASSERT(foundDeclaration);
         bool success = false;
         foreach(ContextBrowserView* contextView, m_views) {
           if(masterWidget(contextView) == masterWidget(view)) {
-              contextView->updateHistory(ctx, position);
               contextView->setDeclaration(foundDeclaration, ctx->topContext());
               success = true;
           }
@@ -524,7 +556,7 @@ bool ContextBrowserPlugin::showDeclarationView(View* view, const SimpleCursor& p
         return success;
 }
 
-bool ContextBrowserPlugin::showSpecialObjectView(View* view, const SimpleCursor& position, ILanguage* pickedLanguage, DUContext* ctx)
+bool ContextBrowserPlugin::showSpecialObjectView(View* view, const SimpleCursor& position, ILanguage* pickedLanguage, DUContext* /*ctx*/)
 {
         //Q_ASSERT(pickedLanguage != 0); // specialObject was found, pickedLanguage must have been set
         if (!pickedLanguage) {
@@ -536,7 +568,6 @@ bool ContextBrowserPlugin::showSpecialObjectView(View* view, const SimpleCursor&
           if(masterWidget(contextView) == masterWidget(view)) {
               ILanguageSupport* ls = pickedLanguage->languageSupport();
               QWidget* w = ls->specialLanguageObjectNavigationWidget(view->document()->url(), position);
-              contextView->updateHistory(ctx, position);
               contextView->setSpecialNavigationWidget(w);
               success = true;
             }
@@ -546,10 +577,10 @@ bool ContextBrowserPlugin::showSpecialObjectView(View* view, const SimpleCursor&
 
 void ContextBrowserPlugin::showContextView(View* view, const SimpleCursor& position, DUContext* ctx)
 {
+    Q_UNUSED(position);
     Q_ASSERT(ctx);
     foreach(ContextBrowserView* contextView, m_views) {
       if(masterWidget(contextView) == masterWidget(view)) {
-        contextView->updateHistory(ctx, position);
         contextView->setContext(SpecializationStore::self().applySpecialization(ctx, ctx->topContext()));
       }
     }
@@ -564,16 +595,40 @@ DUContext* contextAt(const SimpleCursor& position, TopDUContext* topContext)
 }
 
 
-void ContextBrowserPlugin::unHighlightAll()
+void ContextBrowserPlugin::unHighlightAll(KTextEditor::View* selectView)
 {
-    foreach(KTextEditor::View* view, m_highlightedDeclarations.keys())
+    foreach(KTextEditor::View* view, m_highlightedDeclarations.keys()) {
+      if(selectView && selectView != view)
+        continue;
+      
+      KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>(view->document());
+    
+      QMutexLocker smartLock(smart->smartMutex());
+      QMutexLocker lock(&m_backupsMutex);
+
+      QMap< KTextEditor::SmartRange*, QPair< Attribute::Ptr, Attribute::Ptr > >::iterator it = m_backups.begin();
+      while(it != m_backups.end()) {
+        if(it.key()->document() == view->document()) {
+          if(it.key()->attribute() == it->first)
+            it.key()->setAttribute(it->second); //Set the backed up attribute, if it wasn't changed yet
+          ignoreRange(it.key());
+          it = m_backups.erase(it);
+        } else {
+          ++it;
+        }
+      }
+      
       changeHighlight( view, m_highlightedDeclarations[view].data(), false, false );
+    }
 
     m_highlightedDeclarations.clear();
 }
 
 void ContextBrowserPlugin::updateBrowserWidgetFor(View* view)
 {
+    bool keptHighlightedDeclaration = m_keepHighlightedDeclaration.contains(view) && m_highlightedDeclarations[view].data();
+    m_keepHighlightedDeclaration.remove(view);
+    
     bool mouseHighlight =
       (view->document()->url() == m_mouseHoverDocument) &&
       (m_mouseHoverCursor.isValid());
@@ -588,7 +643,9 @@ void ContextBrowserPlugin::updateBrowserWidgetFor(View* view)
     ///First: Check whether there is a special language object
     ///@todo Maybe make this optional, because it can be slow
     ILanguage* pickedLanguage = 0;
-    bool foundSpecialObject = findSpecialObject(view,position, pickedLanguage);
+    bool foundSpecialObject = 0;
+    if(!keptHighlightedDeclaration)
+      foundSpecialObject  = findSpecialObject(view,position, pickedLanguage);
 
     KDevelop::DUChainReadLocker lock( DUChain::lock(), 100 );
     if(!lock.locked()) {
@@ -596,14 +653,39 @@ void ContextBrowserPlugin::updateBrowserWidgetFor(View* view)
       return;
     }
 
-    unHighlightAll();
+    if(!keptHighlightedDeclaration)
+      unHighlightAll();
 
     TopDUContext* topContext = DUChainUtils::standardContextForUrl(view->document()->url());
     if (!topContext) return;
     DUContext* ctx = contextAt(position, topContext);
     if (!ctx) return;
-    Declaration* foundDeclaration = (foundSpecialObject) ? 0 : findDeclaration(view, position, mouseHighlight);
-
+    Declaration* foundDeclaration = 0;
+    
+    //Only update the history if this context is around the text cursor
+    if(core()->documentController()->activeDocument() && position == SimpleCursor(view->cursorPosition()) && view->document() == core()->documentController()->activeDocument()->textDocument())
+    {
+      foreach(ContextBrowserView* contextView, m_views)
+        if(masterWidget(contextView) == masterWidget(view))
+          contextView->updateHistory(ctx, position);
+    }
+    
+    if(keptHighlightedDeclaration) {
+      foundDeclaration = m_highlightedDeclarations[view].data();
+      kDebug() << "keeping";
+    }
+    
+    if(!foundSpecialObject && !foundDeclaration)
+      foundDeclaration = findDeclaration(view, position, mouseHighlight);
+    
+    if( foundDeclaration ) {
+      m_lastHighlightedDeclaration = IndexedDeclaration(foundDeclaration);
+      m_highlightedDeclarations[view] = foundDeclaration;
+      changeHighlight( view, foundDeclaration, true, mouseHighlight );
+    }else{
+      m_highlightedDeclarations.remove(view);
+    }
+    
     bool addedWidget = false;
     // try to add a declaration navigation widget
     if(foundDeclaration || foundSpecialObject) {
@@ -645,6 +727,19 @@ void ContextBrowserPlugin::parseJobFinished(KDevelop::ParseJob* job)
     KDevelop::DUChainReadLocker lock( DUChain::lock() );
     registerAsRangeWatcher(job->duChain());
   }
+  
+  for(QMap< View*, DeclarationPointer >::const_iterator it = m_highlightedDeclarations.begin(); it != m_highlightedDeclarations.end(); ++it) {
+    if(it.key()->document()->url() == job->document().toUrl()) {
+      if(m_updateViews.isEmpty())
+        m_updateTimer->start(highlightingTimeout);
+      
+      if(!m_updateViews.contains(it.key())) {
+        kDebug() << "adding view for update";
+        m_updateViews << it.key();
+        m_keepHighlightedDeclaration.insert(it.key());
+      }
+    }
+  }
 }
 
 void ContextBrowserPlugin::registerAsRangeWatcher(KDevelop::DUChainBase* base)
@@ -716,18 +811,35 @@ void ContextBrowserPlugin::viewDestroyed( QObject* obj )
   m_updateViews.remove(static_cast<View*>(obj));
 }
 
-void ContextBrowserPlugin::cursorPositionChanged( View* view, const KTextEditor::Cursor& /*newPosition*/ )
+void ContextBrowserPlugin::cursorPositionChanged( View* view, const KTextEditor::Cursor& newPosition )
 {
+  if(view->document() == m_lastInsertionDocument && newPosition == m_lastInsertionPos)
+  {
+    //Do not update the highlighting while typing
+    m_lastInsertionDocument = 0;
+    m_lastInsertionPos = KTextEditor::Cursor();
+    m_keepHighlightedDeclaration.insert(view);
+  }else{
+    m_keepHighlightedDeclaration.remove(view);
+  }
   clearMouseHover();
   m_updateViews.insert(view);
   m_updateTimer->start(highlightingTimeout/2); // triggers updateViews()
 }
 
+void ContextBrowserPlugin::textInserted(KTextEditor::Document* doc, KTextEditor::Range range)
+{
+  m_lastInsertionDocument = doc;
+  m_lastInsertionPos = range.end();
+}
+
 void ContextBrowserPlugin::viewCreated( KTextEditor::Document* , View* v )
 {
   disconnect( v, SIGNAL( cursorPositionChanged( KTextEditor::View*, const KTextEditor::Cursor& ) ), this, SLOT( cursorPositionChanged( KTextEditor::View*, const KTextEditor::Cursor& ) ) ); ///Just to make sure that multiple connections don't happen
   connect( v, SIGNAL( cursorPositionChanged( KTextEditor::View*, const KTextEditor::Cursor& ) ), this, SLOT( cursorPositionChanged( KTextEditor::View*, const KTextEditor::Cursor& ) ) );
   connect( v, SIGNAL(destroyed( QObject* )), this, SLOT( viewDestroyed( QObject* ) ) );
+  disconnect( v->document(), SIGNAL(textInserted(KTextEditor::Document*,KTextEditor::Range)), this, SLOT(textInserted(KTextEditor::Document*,KTextEditor::Range)));
+  connect( v->document(), SIGNAL(textInserted(KTextEditor::Document*,KTextEditor::Range)), this, SLOT(textInserted(KTextEditor::Document*,KTextEditor::Range)));
 
   KTextEditor::TextHintInterface *iface = dynamic_cast<KTextEditor::TextHintInterface*>(v);
   if( !iface )
@@ -938,6 +1050,19 @@ void ContextBrowserPlugin::unRegisterToolView(ContextBrowserView* view)
   m_views.removeAll(view);
 }
 
+QWidget* ContextBrowserPlugin::toolbarWidgetForMainWindow(QWidget* widgetInWindow)
+{
+  QWidget* master = masterWidget(widgetInWindow);
+  
+  for(QList< QPointer< QWidget > >::iterator it = m_toolbarWidgets.begin(); it != m_toolbarWidgets.end(); ++it) {
+    if((*it) && masterWidget(*it) == master) {
+      return *it;
+    }
+  }
+  m_toolbarWidgets.append(new QWidget(master));
+  m_toolbarWidgets.back()->setHidden(true);
+  return m_toolbarWidgets.back();
+}
 
 #include "contextbrowser.moc"
 
