@@ -40,6 +40,9 @@
 #include <interfaces/iplugincontroller.h>
 
 #include "executeplugin.h"
+#include <util/kdevstringhandler.h>
+#include <util/environmentgrouplist.h>
+#include <util/projectitemlineedit.h>
 
 KIcon NativeAppConfigPage::icon() const
 {
@@ -48,7 +51,7 @@ KIcon NativeAppConfigPage::icon() const
 
 //TODO: Make sure to auto-add the executable target to the dependencies when its used.
 
-void NativeAppConfigPage::loadFromConfiguration(const KConfigGroup& cfg) 
+void NativeAppConfigPage::loadFromConfiguration(const KConfigGroup& cfg, KDevelop::IProject* project ) 
 {
     bool b = blockSignals( true );
     if( cfg.readEntry( ExecutePlugin::isExecutableEntry, false ) ) 
@@ -57,14 +60,25 @@ void NativeAppConfigPage::loadFromConfiguration(const KConfigGroup& cfg)
         executablePath->setUrl( cfg.readEntry( ExecutePlugin::executableEntry, KUrl() ) );
     } else 
     {
+        if( project )
+        {
+            projectTarget->setBaseItem( project->projectItem() );
+        } else {
+            projectTarget->setBaseItem( 0 );
+        }
         projectTargetRadio->setChecked( true );
-        projectTarget->setText( cfg.readEntry( ExecutePlugin::projectTargetEntry, "" ) );
+        projectTarget->setItemPath( cfg.readEntry( ExecutePlugin::projectTargetEntry, QStringList() ) );
     }
     arguments->setText( cfg.readEntry( ExecutePlugin::argumentsEntry, "" ) );
     workingDirectory->setUrl( cfg.readEntry( ExecutePlugin::workingDirEntry, KUrl() ) );
     environment->setCurrentProfile( cfg.readEntry( ExecutePlugin::environmentGroupEntry, "default" ) );
     runInTerminal->setChecked( cfg.readEntry( ExecutePlugin::useTerminalEntry, false ) );
-    dependencies->addItems( cfg.readEntry( ExecutePlugin::dependencyEntry, QStringList() ) );
+    QVariantList deps = KDevelop::stringToQVariant( cfg.readEntry( ExecutePlugin::dependencyEntry, QString() ) ).toList();
+    QStringList strDeps;
+    foreach( QVariant dep, deps ) {
+        QListWidgetItem* item = new QListWidgetItem( KDevelop::joinWithEscaping( dep.toStringList(), '/', '\\' ), dependencies );
+        item->setData( Qt::UserRole, dep );
+    }
     dependencyAction->setCurrentIndex( dependencyAction->findData( cfg.readEntry( ExecutePlugin::dependencyActionEntry, "Nothing" ) ) );
     blockSignals( b );
 }
@@ -72,11 +86,7 @@ void NativeAppConfigPage::loadFromConfiguration(const KConfigGroup& cfg)
 NativeAppConfigPage::NativeAppConfigPage( QWidget* parent ) 
     : LaunchConfigurationPage( parent )
 {
-    setupUi(this);
-    //Setup completions for the target lineedits
-    projectTarget->setCompleter( new ProjectItemCompleter( KDevelop::ICore::self()->projectController()->projectModel(), this ) );
-    targetDependency->setCompleter( new ProjectItemCompleter( KDevelop::ICore::self()->projectController()->projectModel(), this ) );
-    
+    setupUi(this);    
     //Setup data info for combobox
     dependencyAction->setItemData(0, "Nothing" );
     dependencyAction->setItemData(1, "Build" );
@@ -91,8 +101,12 @@ NativeAppConfigPage::NativeAppConfigPage( QWidget* parent )
     //Set workingdirectory widget to ask for directories rather than files
     workingDirectory->setMode(KFile::Directory | KFile::ExistingOnly | KFile::LocalOnly);
 
+    KDevelop::EnvironmentGroupList env( KGlobal::config() );
+    environment->addItems( env.groups() );
+
+
     //connect signals to changed signal
-    connect( projectTarget, SIGNAL(textEdited(const QString&)), SIGNAL(changed()) );
+    connect( projectTarget, SIGNAL(textChanged(const QString&)), SIGNAL(changed()) );
     connect( projectTargetRadio, SIGNAL(toggled(bool)), SIGNAL(changed()) );
     connect( executableRadio, SIGNAL(toggled(bool)), SIGNAL(changed()) );
     connect( executablePath->lineEdit(), SIGNAL(textEdited(const QString&)), SIGNAL(changed()) );
@@ -113,7 +127,7 @@ NativeAppConfigPage::NativeAppConfigPage( QWidget* parent )
     connect( dependencyAction, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
     connect( runInTerminal, SIGNAL(toggled(bool)), SIGNAL(changed()) );
     connect( dependencyAction, SIGNAL(currentIndexChanged(int)), SLOT(activateDeps(int)) );
-    connect( targetDependency, SIGNAL(textEdited(QString)), SLOT(depEdited(QString)));
+    connect( targetDependency, SIGNAL(textChanged(QString)), SLOT(depEdited(QString)));
 }
 
 
@@ -135,6 +149,7 @@ void NativeAppConfigPage::activateDeps( int idx )
 
 void NativeAppConfigPage::checkActions( const QItemSelection& selected, const QItemSelection& unselected )
 {
+    Q_UNUSED( unselected );
     kDebug() << "checkActions";
     if( !selected.indexes().isEmpty() )
     {
@@ -184,7 +199,8 @@ void NativeAppConfigPage::moveDependencyUp()
 
 void NativeAppConfigPage::addDep()
 {
-    dependencies->addItem( targetDependency->text() );
+    QListWidgetItem* item = new QListWidgetItem( targetDependency->text(), dependencies );
+    item->setData( Qt::UserRole, targetDependency->itemPath() );
     targetDependency->setText("");
     addDependency->setEnabled( false );
     dependencies->selectionModel()->select( dependencies->model()->index( dependencies->model()->rowCount() - 1, 0, QModelIndex() ), QItemSelectionModel::ClearAndSelect | QItemSelectionModel::SelectCurrent );
@@ -203,8 +219,9 @@ void NativeAppConfigPage::removeDep()
     }
 }
 
-void NativeAppConfigPage::saveToConfiguration(KConfigGroup cfg) const
+void NativeAppConfigPage::saveToConfiguration( KConfigGroup cfg, KDevelop::IProject* project ) const
 {
+    Q_UNUSED( project );
     cfg.writeEntry( ExecutePlugin::isExecutableEntry, executableRadio->isChecked() );
     if( executableRadio-> isChecked() )
     {
@@ -212,7 +229,7 @@ void NativeAppConfigPage::saveToConfiguration(KConfigGroup cfg) const
         cfg.deleteEntry( ExecutePlugin::projectTargetEntry );
     } else
     {
-        cfg.writeEntry( ExecutePlugin::projectTargetEntry, projectTarget->text() );
+        cfg.writeEntry( ExecutePlugin::projectTargetEntry, projectTarget->itemPath() );
         cfg.deleteEntry( ExecutePlugin::executableEntry );
     }
     cfg.writeEntry( ExecutePlugin::argumentsEntry, arguments->text() );
@@ -220,12 +237,12 @@ void NativeAppConfigPage::saveToConfiguration(KConfigGroup cfg) const
     cfg.writeEntry( ExecutePlugin::environmentGroupEntry, environment->currentProfile() );
     cfg.writeEntry( ExecutePlugin::useTerminalEntry, runInTerminal->isChecked() );
     cfg.writeEntry( ExecutePlugin::dependencyActionEntry, dependencyAction->itemData( dependencyAction->currentIndex() ).toString() );
-    QStringList deps;
+    QVariantList deps;
     for( int i = 0; i < dependencies->count(); i++ )
     {
-        deps << dependencies->item( i )->text();
+        deps << dependencies->item( i )->data( Qt::UserRole );
     }
-    cfg.writeEntry( ExecutePlugin::dependencyEntry, deps );
+    cfg.writeEntry( ExecutePlugin::dependencyEntry, KDevelop::qvariantToString( QVariant( deps ) ) );
 }
 
 QString NativeAppConfigPage::title() const 
