@@ -40,30 +40,41 @@
 
 #include <QtDesigner/QExtensionFactory>
 #include <QtCore/QRegExp>
+#include <QtCore/QFileInfo>
 
 #include <KIO/Job>
 #include <KIO/NetAccess>
 
-///FIXME: remove the hack and these includes
-#include <interfaces/isession.h>
-#include <language/backgroundparser/parseprojectjob.h>
-///END
+#include "genericmanagerlistjob.h"
 
 K_PLUGIN_FACTORY(GenericSupportFactory, registerPlugin<GenericProjectManager>(); )
 K_EXPORT_PLUGIN(GenericSupportFactory(KAboutData("kdevgenericmanager","kdevgenericprojectmanager",ki18n("Generic Project Manager"), "0.1", ki18n("A plugin to support basic project management on a filesystem level"), KAboutData::License_GPL)))
 
 class GenericProjectManagerPrivate
 {
+public:
+    GenericProjectManagerPrivate()
+    {}
 };
 
+
+/**
+ * Reads project config and returns lists for include and exclude rules.
+ */
+GenericProjectManager::IncludeRules getIncludeRules(KDevelop::IProject* project) {
+    KConfigGroup filtersConfig = project->projectConfiguration()->group("Filters");
+    QStringList includes = filtersConfig.readEntry("Includes", QStringList("*"));
+    ///TODO: filter hidden files by default
+    QStringList excludes = filtersConfig.readEntry("Excludes", QStringList("*/.*"));
+
+    return qMakePair(includes, excludes);
+}
+
 GenericProjectManager::GenericProjectManager( QObject *parent, const QVariantList & args )
         : KDevelop::IPlugin( GenericSupportFactory::componentData(), parent ), KDevelop::IProjectFileManager(), d( new GenericProjectManagerPrivate )
 {
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IProjectFileManager )
     Q_UNUSED( args )
-    
-    connect( KDevelop::ICore::self()->projectController(), SIGNAL(projectOpened(KDevelop::IProject*)),
-             this, SLOT(waitForProjectOpen(KDevelop::IProject*)) );
 }
 
 GenericProjectManager::~GenericProjectManager()
@@ -72,7 +83,7 @@ GenericProjectManager::~GenericProjectManager()
 }
 
 bool GenericProjectManager::isValid( const KUrl &url, const bool isFolder, KDevelop::IProject* project,
-                                     const QStringList &includes, const QStringList &excludes ) const
+                                     const IncludeRules &rules ) const
 {
     if ( isFolder && ( url.fileName() == "." || url.fileName() == ".."
             || ( url.fileName() == ".kdev4" && url.upUrl() == project->folder() ) ) ) {
@@ -90,7 +101,7 @@ bool GenericProjectManager::isValid( const KUrl &url, const bool isFolder, KDeve
         isFolder ? KUrl::AddTrailingSlash : KUrl::RemoveTrailingSlash
     );
 
-    for ( QStringList::ConstIterator it = includes.constBegin(); !ok && it != includes.constEnd(); ++it ) {
+    for ( QStringList::ConstIterator it = rules.first.constBegin(); !ok && it != rules.first.constEnd(); ++it ) {
         QRegExp rx( *it, Qt::CaseSensitive, QRegExp::Wildcard );
         if ( rx.exactMatch( relativePath ) ) {
             ok = true;
@@ -102,7 +113,7 @@ bool GenericProjectManager::isValid( const KUrl &url, const bool isFolder, KDeve
         return false;
     }
 
-    for ( QStringList::ConstIterator it = excludes.constBegin(); it != excludes.constEnd(); ++it ) {
+    for ( QStringList::ConstIterator it = rules.second.constBegin(); it != rules.second.constEnd(); ++it ) {
         QRegExp rx( *it, Qt::CaseSensitive, QRegExp::Wildcard );
         if ( rx.exactMatch( relativePath ) ) {
             return false;
@@ -120,98 +131,67 @@ QList<KDevelop::ProjectFolderItem*> GenericProjectManager::parse( KDevelop::Proj
     return QList<KDevelop::ProjectFolderItem*>();
 }
 
-void GenericProjectManager::waitForProjectOpen(KDevelop::IProject* project)
+KJob* GenericProjectManager::eventuallyReadFolder( KDevelop::ProjectFolderItem* item )
 {
-    if ( project->managerPlugin() == this ) {
-        kDebug() << "generic project got opened:" << project->folder();
-        eventuallyReadFolder( project->projectItem() );
-    }
+    GenericManagerListJob* listJob = new GenericManagerListJob( item );
+    kDebug() << "adding job" << listJob << item->url() << "for project" << item->project();
+
+    KDevelop::ICore::self()->runController()->registerJob( listJob );
+
+    connect( listJob, SIGNAL(entries(KDevelop::IProject*,KUrl,KIO::UDSEntryList)),
+             this, SLOT(addJobItems(KDevelop::IProject*,KUrl, KIO::UDSEntryList)) );
+
+    connect( this, SIGNAL(appendSubDir(KDevelop::ProjectFolderItem*)),
+             listJob, SLOT(addSubDir(KDevelop::ProjectFolderItem*)));
+
+    return listJob;
 }
 
-void GenericProjectManager::eventuallyReadFolder( KDevelop::ProjectFolderItem* item )
-{
-    ///TODO: listRecursive does not follow directory links!
-    KIO::ListJob* job = KIO::listRecursive( item->url(), KIO::HideProgressInfo );
-    kDebug() << "adding job" << job << job->url() << "for project" << item->project();
-
-    KDevelop::ICore::self()->runController()->registerJob( job );
-
-    connect( job, SIGNAL(entries(KIO::Job*, KIO::UDSEntryList)),
-                this, SLOT(addJobItems(KIO::Job*, KIO::UDSEntryList)) );
-
-    /// FIXME: once async project managers are supported, we won't have to do this hack
-    /// since parse() cannot return anything the parseprojectjob gets a list of empty files
-    /// => hence this hack at least starts the parsepojrectjob if required
-    if ( item->isProjectRoot() &&
-         KDevelop::ICore::self()->activeSession()->config()->group( "Project Manager" ).readEntry( "Parse All Project Sources", true ) )
-    {
-        connect( job, SIGNAL(result(KJob*)),
-                this, SLOT(startProjectParseJob(KJob*)) );
-    }
-}
-
-void GenericProjectManager::startProjectParseJob( KJob* j )
-{
-    if ( j->error() ) {
-        return;
-    }
-    KIO::SimpleJob* job(dynamic_cast<KIO::SimpleJob*>(j));
-    Q_ASSERT(job);
-
-    KDevelop::IProject *project = KDevelop::ICore::self()->projectController()->findProjectForUrl(job->url());
-    if ( !project ) {
-        return;
-    }
-    KJob* parseProjectJob = new KDevelop::ParseProjectJob(project);
-    KDevelop::ICore::self()->runController()->registerJob(parseProjectJob);
-}
-
-void GenericProjectManager::addJobItems(KIO::Job* j, KIO::UDSEntryList entries)
+void GenericProjectManager::addJobItems(KDevelop::IProject* project, const KUrl& listedDirUrl, KIO::UDSEntryList entries)
 {
     if ( entries.empty() ) {
         return;
     }
-
-    KIO::SimpleJob* job(dynamic_cast<KIO::SimpleJob*>(j));
-    Q_ASSERT(job);
+    if ( !project ) {
+        kDebug() << "no project found for" << listedDirUrl;
+        return;
+    }
 
     // the folder we are currently adding items to
-    // the UDS_NAME is essentially a path relative to the job->url()
-    KUrl curUrl = job->url();
+    // the UDS_NAME is essentially a path relative to listedDirUrl
+    KUrl curUrl = listedDirUrl;
     curUrl.addPath( entries.first().stringValue( KIO::UDSEntry::UDS_NAME ) );
     curUrl.setFileName( "" );
 
     kDebug() << "reading contents of " << curUrl << "total entries:" << entries.count();
 
-    KDevelop::IProject* project = KDevelop::ICore::self()->projectController()->findProjectForUrl( job->url() );
-    if ( !project ) {
-        kDebug() << "no project found for" << job->url();
-        return;
-    }
-
     QList<KDevelop::ProjectFolderItem*> fitems = project->foldersForUrl( curUrl );
     if ( fitems.isEmpty() ) {
         kDebug() << "parent was filtered";
         return;
     }
     KDevelop::ProjectFolderItem* item = fitems.first();
-
-    KConfigGroup filtersConfig = project->projectConfiguration()->group("Filters");
-    QStringList includes = filtersConfig.readEntry("Includes", QStringList("*"));
-    ///TODO: filter hidden files by default
-    QStringList excludes = filtersConfig.readEntry("Excludes", QStringList("*/.*"));
+    
+    const IncludeRules rules = getIncludeRules(project);
 
     // build lists of valid files and folders with relative urls to the project folder
     KUrl::List files;
     KUrl::List folders;
     foreach ( KIO::UDSEntry entry, entries ) {
-        KUrl url = job->url();
+        KUrl url = listedDirUrl;
         url.addPath( entry.stringValue( KIO::UDSEntry::UDS_NAME ) );
 
-        if ( !isValid( url, entry.isDir(), project, includes, excludes ) ) {
+        if ( !isValid( url, entry.isDir(), project, rules ) ) {
             continue;
         } else {
             if ( entry.isDir() ) {
+                if( entry.isLink() ) {
+                    KUrl linkedUrl = listedDirUrl;
+                    linkedUrl.cd(entry.stringValue( KIO::UDSEntry::UDS_LINK_DEST ));
+                    // make sure we don't end in an infinite loop
+                    if( linkedUrl.isParentOf( project->folder() ) || project->folder().isParentOf( linkedUrl ) || linkedUrl == project->folder() )
+                        continue;
+                }
                 folders << url;
             } else {
                 files << url;
@@ -230,6 +210,7 @@ void GenericProjectManager::addJobItems(KIO::Job* j, KIO::UDSEntryList entries)
             int index = folders.indexOf( f->folderName() );
             if ( index == -1 ) {
                 // folder got removed or is now invalid
+                kDebug() << "removing folder:" << f->url();
                 item->removeRow( j );
                 --j;
             } else {
@@ -242,6 +223,7 @@ void GenericProjectManager::addJobItems(KIO::Job* j, KIO::UDSEntryList entries)
             int index = files.indexOf( f->fileName() );
             if ( index == -1 ) {
                 // file got removed or is now invalid
+                kDebug() << "removing file:" << f->url();
                 project->removeFromFileSet( KDevelop::IndexedString( f->url() ) );
                 item->removeRow( j );
                 --j;
@@ -253,19 +235,18 @@ void GenericProjectManager::addJobItems(KIO::Job* j, KIO::UDSEntryList entries)
     }
 
     // add new rows
-    ///TODO: why do the fileitems look like remote folders, and why is the sorting fubar?
     foreach ( const KUrl& url, files ) {
         new KDevelop::ProjectFileItem( project, url, item );
         project->addToFileSet( KDevelop::IndexedString( url ) );
     }
     foreach ( const KUrl& url, folders ) {
-        new KDevelop::ProjectFolderItem( project, url, item );
+        emit appendSubDir( new KDevelop::ProjectFolderItem( project, url, item ) );
     }
 }
 
 KDevelop::ProjectFolderItem *GenericProjectManager::import( KDevelop::IProject *project )
 {
-    KDevelop::ProjectFolderItem *projectRoot=new KDevelop::ProjectFolderItem( project, project->folder(), 0 );
+    KDevelop::ProjectFolderItem *projectRoot = new KDevelop::ProjectFolderItem( project, project->folder(), 0 );
     kDebug() << "imported new project" << project->name() << "at" << projectRoot->url();
     projectRoot->setProjectRoot(true);
 
@@ -273,14 +254,22 @@ KDevelop::ProjectFolderItem *GenericProjectManager::import( KDevelop::IProject *
     if ( project->folder().isLocalFile() ) {
         m_watchers[project] = new KDirWatch( project );
 
-        connect(m_watchers[project], SIGNAL(dirty(QString)), this, SLOT(dirty(QString)));
+        connect(m_watchers[project], SIGNAL(created(QString)),
+                    this, SLOT(created(QString)));
+        connect(m_watchers[project], SIGNAL(deleted(QString)),
+                    this, SLOT(deleted(QString)));
 
-        m_watchers[project]->addDir(project->folder().toLocalFile(), KDirWatch::WatchSubDirs );
+        m_watchers[project]->addDir(project->folder().toLocalFile(), KDirWatch::WatchSubDirs | KDirWatch:: WatchFiles );
     }
 
     return projectRoot;
 }
 
+KJob* GenericProjectManager::createImportJob(KDevelop::ProjectFolderItem* item)
+{
+    return eventuallyReadFolder(item);
+}
+
 bool GenericProjectManager::reload( KDevelop::ProjectBaseItem* item )
 {
     kDebug() << "reloading project" << item->project()->name() << item->project()->folder();
@@ -289,17 +278,46 @@ bool GenericProjectManager::reload( KDevelop::ProjectBaseItem* item )
     return true;
 }
 
-void GenericProjectManager::dirty( const QString &fileName )
+void GenericProjectManager::created(const QString &path)
 {
-    foreach ( KDevelop::IProject* p, KDevelop::ICore::self()->projectController()->projects() ) {
-        foreach ( KDevelop::ProjectFolderItem* item, p->foldersForUrl( KUrl(fileName) ) ) {
-            kDebug() << "reloading item" << item->url();
+    QFileInfo info(path);
 
-            eventuallyReadFolder( item );
+    KUrl url = KUrl(path);
+    KUrl parent = url.upUrl();
+
+    foreach ( KDevelop::IProject* p, m_watchers.keys() ) {
+        if ( !isValid(url, info.isDir(), p, getIncludeRules(p)) ) {
+            continue;
+        }
+        foreach ( KDevelop::ProjectFolderItem* parentItem, p->foldersForUrl(parent) ) {
+            if ( info.isDir() ) {
+                eventuallyReadFolder(new KDevelop::ProjectFolderItem( p, url, parentItem ));
+            } else {
+                new KDevelop::ProjectFileItem( p, url, parentItem );
+                p->addToFileSet( KDevelop::IndexedString( url ) );
+            }
         }
     }
 }
 
+void GenericProjectManager::deleted(const QString &path)
+{
+    QFileInfo info(path);
+
+    KUrl url = KUrl(path);
+
+    foreach ( KDevelop::IProject* p, m_watchers.keys() ) {
+        if ( info.isDir() ) {
+            foreach ( KDevelop::ProjectFolderItem* item, p->foldersForUrl(url) ) {
+                item->parent()->removeRow(item->row());
+            }
+        } else {
+            foreach ( KDevelop::ProjectFileItem* item, p->filesForUrl(url) ) {
+                item->parent()->removeRow(item->row());
+            }
+        }
+    }
+}
 
 KDevelop::ProjectFolderItem* GenericProjectManager::addFolder( const KUrl& url,
         KDevelop::ProjectFolderItem * folder )
