@@ -75,6 +75,10 @@ public:
     virtual QList<KDevelop::ProjectFolderItem*> parse( KDevelop::ProjectFolderItem *item );
     virtual KDevelop::ProjectFolderItem *import( KDevelop::IProject *project );
     virtual bool reload(KDevelop::ProjectBaseItem* item);
+    virtual KJob* createImportJob(KDevelop::ProjectFolderItem* item);
+
+    /// first item is includes, second excludes
+    typedef QPair<QStringList, QStringList> IncludeRules;
 
 Q_SIGNALS:
     void projectItemConfigWidget(const QList<KDevelop::ProjectBaseItem*> &dom, KDialogBase *dialog);
@@ -88,27 +92,23 @@ Q_SIGNALS:
     void fileRemoved(KDevelop::ProjectFileItem* file);
     void fileRenamed(const KUrl& oldFile,
                      KDevelop::ProjectFileItem* newFile);
+    void appendSubDir( KDevelop::ProjectFolderItem* item );
 
 private Q_SLOTS:
-    void eventuallyReadFolder( KDevelop::ProjectFolderItem* item );
-    ///NOTE: make sure the project for the job's URL is already registered at the controller
-    void addJobItems(KIO::Job* j, KIO::UDSEntryList entries);
-    void waitForProjectOpen( KDevelop::IProject* project );
+    KJob* eventuallyReadFolder( KDevelop::ProjectFolderItem* item );
+    void addJobItems(KDevelop::IProject* project, const KUrl& url, KIO::UDSEntryList entries);
 
-    ///FIXME: remove this hack once async managers are better supported
-    void startProjectParseJob(KJob* j);
+    void deleted(const QString &path);
+    void created(const QString &path);
 
 private:
     bool isValid( const KUrl& url, const bool isFolder, KDevelop::IProject* project,
-                  const QStringList& includes, const QStringList& excludes ) const;
+                  const IncludeRules& rules ) const;
     QMap<KDevelop::IProject*, KDirWatch*> m_watchers;
     QMap<KIO::Job*, KDevelop::IProject*> m_jobProjects;
 
-private Q_SLOTS:
-    void dirty( const QString &fileName );
-
 private:
-    struct GenericProjectManagerPrivate* const d;
+    class GenericProjectManagerPrivate* const d;
 };
 
 #endif // KDEVGENERICIMPORTER_H
