@@ -26,8 +26,8 @@
 
 #include <qtest_kde.h>
 #include <QtTest/QtTest>
-#include <shell/testcore.h>
-#include <tests/common/autotestshell.h>
+#include <tests/testcore.h>
+#include <tests/autotestshell.h>
 
 #include <KUrl>
 #include <KDebug>
