@@ -66,33 +66,18 @@ std::string*/
 #include <kiconeffect.h>
 #include <kcolorutils.h>
 #include <kcolorscheme.h>
-
-QStringList splitArgs( const QString& str ) {
-    QStringList ret;
-    QString current = str;
-    int pos = 0;
-    while ( ( pos = current.indexOf( ' ', pos ) ) != -1 ) {
-        if ( current[ 0 ] == '"' ) {
-            int end = current.indexOf( '"' );
-            if ( end > pos )
-                pos = end;
-        }
-        QString s = current.left( pos );
-        if ( s.length() > 0 )
-            ret << s;
-        current = current.mid( pos + 1 );
-        pos = 0;
-    }
-    if ( current.length() )
-        ret << current;
-    return ret;
-}
+#include <sublime/controller.h>
+#include <sublime/mainwindow.h>
+#include <sublime/area.h>
+#include <interfaces/iprojectcontroller.h>
 
 using namespace KDevelop;
 
 Q_DECLARE_METATYPE( const Diff2::DiffModel* )
 
-PatchReviewToolView::PatchReviewToolView( QWidget* parent, PatchReviewPlugin* plugin ) : QWidget( parent ), m_plugin( plugin ), m_reversed( false ) {
+PatchReviewToolView::PatchReviewToolView( QWidget* parent, PatchReviewPlugin* plugin ) 
+: QWidget( parent ), m_reversed( false ), m_plugin( plugin ) 
+{
     showEditDialog();
     connect( plugin, SIGNAL(patchChanged()), SLOT(patchChanged()) );
     patchChanged();
@@ -110,45 +95,100 @@ PatchReviewToolView::~PatchReviewToolView() {
 
 void PatchReviewToolView::updatePatchFromEdit() {
 
-    LocalPatchSourcePointer ps = m_plugin->patch();
+    IPatchSource::Ptr ips = m_plugin->patch();
 
-    if ( !ps )
+    if ( !ips )
         return;
+    LocalPatchSource* lpatch = dynamic_cast<LocalPatchSource*>(ips.data());
+    if(!lpatch)
+      return;
 
-    ps->command = m_editPatch.command->text();
-    ps->filename = m_editPatch.filename->url();
-    ps->baseDir = m_editPatch.baseDir->url();
-    ps->depth = m_editPatch.depth->value();
+    lpatch->m_command = m_editPatch.command->text();
+    lpatch->m_filename = m_editPatch.filename->url();
+    lpatch->m_baseDir = m_editPatch.baseDir->url();
+//     lpatch->m_depth = m_editPatch.depth->value();
 
     m_plugin->notifyPatchChanged();
 }
 
 void PatchReviewToolView::fillEditFromPatch() {
 
-    LocalPatchSourcePointer patch = m_plugin->patch();
-    if ( !patch )
+    IPatchSource::Ptr ipatch = m_plugin->patch();
+    if ( !ipatch )
         return ;
+    
+    disconnect( m_editPatch.patchSelection, SIGNAL(currentIndexChanged(int)), this, SLOT(patchSelectionChanged(int)));
+    
+    m_editPatch.patchSelection->clear();
+    foreach(IPatchSource::Ptr patch, m_plugin->knownPatches())
+    {
+      if(!patch)
+        continue;
+      m_editPatch.patchSelection->addItem(patch->icon(), patch->name());
+      if(patch == ipatch)
+        m_editPatch.patchSelection->setCurrentIndex(m_editPatch.patchSelection->count()-1);
+    }
+    
+    connect( m_editPatch.patchSelection, SIGNAL(currentIndexChanged(int)), this, SLOT(patchSelectionChanged(int)));
 
-    m_editPatch.command->setText( patch->command );
-    m_editPatch.filename->setUrl( patch->filename );
-    m_editPatch.baseDir->setUrl( patch->baseDir );
-    m_editPatch.depth->setValue( patch->depth );
+    m_editPatch.cancelReview->setVisible(ipatch->canCancel());
 
-//   slotStateChanged();
+    QString finishText = i18n("Finish Review");
+    if(!ipatch->finishReviewCustomText().isEmpty())
+      finishText = ipatch->finishReviewCustomText();
+    kDebug() << "finish-text: " << finishText;
+    m_editPatch.finishReview->setText(finishText);
+    
+    if(m_customWidget) {
+      kDebug() << "removing custom widget";
+      m_customWidget->hide();
+      m_editPatch.verticalLayout->removeWidget(m_customWidget);
+    }
 
-    if ( patch->command.isEmpty() )
+    m_customWidget = ipatch->customWidget();
+    if(m_customWidget) {
+      m_editPatch.verticalLayout->insertWidget(0, m_customWidget);
+      m_customWidget->show();
+      kDebug() << "got custom widget";
+    }
+    
+    LocalPatchSource* lpatch = dynamic_cast<LocalPatchSource*>(ipatch.data());
+    if(!lpatch) {
+      m_editPatch.tabWidget->hide();
+      m_editPatch.baseDir->hide();
+      m_editPatch.label->hide();
+      return;
+    }else{
+      m_editPatch.tabWidget->show();
+      m_editPatch.baseDir->show();
+      m_editPatch.label->show();
+    }
+    
+    m_editPatch.command->setText( lpatch->m_command );
+    m_editPatch.filename->setUrl( lpatch->m_filename );
+    m_editPatch.baseDir->setUrl( lpatch->m_baseDir );
+//     m_editPatch.depth->setValue( lpatch->m_depth );
+
+    if ( lpatch->m_command.isEmpty() )
         m_editPatch.tabWidget->setCurrentIndex( m_editPatch.tabWidget->indexOf( m_editPatch.fileTab ) );
     else
         m_editPatch.tabWidget->setCurrentIndex( m_editPatch.tabWidget->indexOf( m_editPatch.commandTab ) );
 }
 
+void PatchReviewToolView::patchSelectionChanged(int selection)
+{
+    if(selection >= 0 && selection < m_plugin->knownPatches().size()) {
+      m_plugin->setPatch(m_plugin->knownPatches()[selection]);
+    }
+}
+
 void PatchReviewToolView::slotEditCommandChanged() {
-    m_editPatch.filename->lineEdit()->setText( "" );
+//     m_editPatch.filename->lineEdit()->setText( "" );
     updatePatchFromEdit();
 }
 
 void PatchReviewToolView::slotEditFileNameChanged() {
-    m_editPatch.command->setText( "" );
+//     m_editPatch.command->setText( "" );
     updatePatchFromEdit();
 }
 
@@ -156,23 +196,28 @@ void PatchReviewToolView::showEditDialog() {
 
     m_editPatch.setupUi( this );
 
-    m_filesModel = new QStandardItemModel( m_editPatch.filesList );
-    m_editPatch.filesList->setModel( m_filesModel );
-
+    m_editPatch.filesList->header()->hide();
+    m_editPatch.filesList->setRootIsDecorated(false);
+    
     m_editPatch.previousHunk->setIcon(KIcon("arrow-up"));
     m_editPatch.nextHunk->setIcon(KIcon("arrow-down"));
     
     connect( m_editPatch.previousHunk, SIGNAL( clicked( bool ) ), this, SLOT( prevHunk() ) );
     connect( m_editPatch.nextHunk, SIGNAL( clicked( bool ) ), this, SLOT( nextHunk() ) );
     connect( m_editPatch.filesList, SIGNAL( doubleClicked( const QModelIndex& ) ), this, SLOT( fileDoubleClicked( const QModelIndex& ) ) );
+    
+    connect( m_editPatch.cancelReview, SIGNAL(clicked(bool)), m_plugin, SLOT(cancelReview()) );
+    connect( m_editPatch.finishReview, SIGNAL(clicked(bool)), this, SLOT(finishReview()) );
     //connect( m_editPatch.cancelButton, SIGNAL( pressed() ), this, SLOT( slotEditCancel() ) );
 
-    connect( this, SIGNAL( finished( int ) ), this, SLOT( slotEditDialogFinished( int ) ) );
+    //connect( this, SIGNAL( finished( int ) ), this, SLOT( slotEditDialogFinished( int ) ) );
 
-    connect( m_editPatch.depth, SIGNAL(valueChanged(int)), SLOT(updatePatchFromEdit()) );
+//     connect( m_editPatch.depth, SIGNAL(valueChanged(int)), SLOT(updatePatchFromEdit()) );
     connect( m_editPatch.filename, SIGNAL( textChanged( const QString& ) ), SLOT(slotEditFileNameChanged()) );
     connect( m_editPatch.baseDir, SIGNAL(textChanged(QString)), SLOT(updatePatchFromEdit()) );
 
+    
+    
     m_editPatch.baseDir->setMode(KFile::Directory);
 
     connect( m_editPatch.command, SIGNAL( textChanged( const QString& ) ), this, SLOT(slotEditCommandChanged()) );
@@ -180,10 +225,11 @@ void PatchReviewToolView::showEditDialog() {
 
     connect( m_editPatch.filename->lineEdit(), SIGNAL( returnPressed() ), this, SLOT(slotEditFileNameChanged()) );
     connect( m_editPatch.filename->lineEdit(), SIGNAL( editingFinished() ), this, SLOT(slotEditFileNameChanged()) );
-    connect( m_editPatch.filename, SIGNAL( urlSelected( const QString& ) ), this, SLOT(slotEditFileNameChanged()) );
+    connect( m_editPatch.filename, SIGNAL( urlSelected( const KUrl& ) ), this, SLOT(slotEditFileNameChanged()) );
     connect( m_editPatch.command, SIGNAL(textChanged(QString)), this, SLOT(slotEditCommandChanged()) );
-    connect( m_editPatch.commandToFile, SIGNAL(clicked(bool)), m_plugin, SLOT(commandToFile()) );
+//     connect( m_editPatch.commandToFile, SIGNAL(clicked(bool)), m_plugin, SLOT(commandToFile()) );
 
+    connect( m_editPatch.patchSelection, SIGNAL(currentIndexChanged(int)), this, SLOT(patchSelectionChanged(int)));
     
     connect( m_editPatch.updateButton, SIGNAL(clicked(bool)), m_plugin, SLOT(forceUpdate()) );
 
@@ -206,10 +252,7 @@ void PatchReviewToolView::prevHunk() {
 
 KUrl PatchReviewPlugin::diffFile()
 {
-    if(!m_patch->filename.isEmpty())
-      return m_patch->filename;
-    else
-      return m_diffFile;
+    return m_patch->file();
 }
 
 void PatchReviewPlugin::seekHunk( bool forwards, const KUrl& fileName ) {
@@ -223,7 +266,7 @@ void PatchReviewPlugin::seekHunk( bool forwards, const KUrl& fileName ) {
             if ( !model || !model->differences() )
                 continue;
 
-            KUrl file = m_patch->baseDir;
+            KUrl file = m_patch->baseDir();
             
             file.addPath( model->sourcePath() );
             file.addPath( model->sourceFile() );
@@ -233,21 +276,25 @@ void PatchReviewPlugin::seekHunk( bool forwards, const KUrl& fileName ) {
 
             IDocument* doc = ICore::self()->documentController()->documentForUrl( file );
 
-            if ( doc && doc == ICore::self()->documentController()->activeDocument() ) {
+            if ( doc && doc == ICore::self()->documentController()->activeDocument() && m_highlighters.contains(doc->url()) && m_highlighters[doc->url()] ) {
+              
                 ICore::self()->documentController()->activateDocument( doc );
                 if ( doc->textDocument() ) {
+                  
+                    KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>( doc->textDocument() );
+                    Q_ASSERT(smart);
+                    QMutexLocker lock(smart->smartMutex());
+
+                    QList< KTextEditor::SmartRange* > ranges = m_highlighters[doc->url()]->ranges();
+                    
                     KTextEditor::View * v = doc->textDocument() ->activeView();
                     int bestLine = -1;
                     if ( v ) {
                         KTextEditor::Cursor c = v->cursorPosition();
-                        for ( Diff2::DifferenceList::const_iterator it = model->differences() ->begin(); it != model->differences() ->end(); ++it ) {
+                        for ( QList< KTextEditor::SmartRange* >::const_iterator it = ranges.begin(); it != ranges.end(); ++it ) {
                             int line;
-                            Diff2::Difference* diff = *it;
                             
-                            line = diff->sourceLineNumber();
-
-                            if ( line > 0 )
-                                line -= 1;
+                            line = (*it)->start().line();
 
                             if ( forwards ) {
                                 if ( line > c.line() && ( bestLine == -1 || line < bestLine ) )
@@ -258,6 +305,7 @@ void PatchReviewPlugin::seekHunk( bool forwards, const KUrl& fileName ) {
                             }
                         }
                         if ( bestLine != -1 ) {
+                          lock.unlock();
                             v->setCursorPosition( KTextEditor::Cursor( bestLine, 0 ) );
                             return ;
                         }
@@ -287,7 +335,7 @@ void PatchReviewPlugin::addHighlighting(const KUrl& highlightFile, IDocument* do
             if ( !model )
                 continue;
 
-            KUrl file = m_patch->baseDir;
+            KUrl file = m_patch->baseDir();
             
             file.addPath( model->sourcePath() );
             file.addPath( model->sourceFile() );
@@ -328,7 +376,7 @@ void PatchReviewPlugin::highlightPatch() {
             if ( !model )
                 continue;
 
-            KUrl file = m_patch->baseDir;
+            KUrl file = m_patch->baseDir();
             
             file.addPath( model->sourcePath() );
             file.addPath( model->sourceFile() );
@@ -343,18 +391,52 @@ void PatchReviewPlugin::highlightPatch() {
     }
 }
 
+
+void PatchReviewToolView::finishReview()
+{
+    QList<KUrl> selectedUrls;
+    for(uint a = 0; a< m_editPatch.filesList->topLevelItemCount(); ++a) {
+      QTreeWidgetItem* item = m_editPatch.filesList->topLevelItem(a);
+      if(item && item->checkState(0) == Qt::Checked) {
+        QVariant v = item->data(0, Qt::UserRole);
+        if( v.canConvert<KUrl>() ) {
+          selectedUrls << v.value<KUrl>();
+        }else if ( v.canConvert<const Diff2::DiffModel*>() ) {
+          const Diff2::DiffModel* model = v.value<const Diff2::DiffModel*>();
+
+          KUrl file = m_plugin->patch()->baseDir();
+          
+          file.addPath( model->sourcePath() );
+          file.addPath( model->sourceFile() );
+          
+          selectedUrls << file;
+        }
+      }
+    }
+    kDebug() << "finishing review with" << selectedUrls;
+    m_plugin->finishReview(selectedUrls);
+}
+
 void PatchReviewToolView::fileDoubleClicked( const QModelIndex& i ) {
     try {
         if ( !m_plugin->modelList() )
             throw "no model";
-        QVariant v = m_filesModel->data( i, Qt::UserRole );
+        
+        QVariant v = i.data( Qt::UserRole );
+        
+        if( v.canConvert<KUrl>() ) {
+          KUrl u = v.value<KUrl>();
+          ICore::self()->documentController()->openDocument( u, KTextEditor::Cursor() );
+          return;
+        }
+        
         if ( !v.canConvert<const Diff2::DiffModel*>() )
             throw "cannot convert";
         const Diff2::DiffModel* model = v.value<const Diff2::DiffModel*>();
         if ( !model )
             throw "bad model-value";
 
-        KUrl file = m_plugin->patch()->baseDir;
+        KUrl file = m_plugin->patch()->baseDir();
         
         file.addPath( model->sourcePath() );
         file.addPath( model->sourceFile() );
@@ -373,7 +455,7 @@ void PatchReviewToolView::fileDoubleClicked( const QModelIndex& i ) {
 
 KUrl PatchReviewToolView::urlForFileModel(const Diff2::DiffModel* model)
 {
-  KUrl file = m_plugin->patch()->baseDir;
+  KUrl file = m_plugin->patch()->baseDir();
   
   file.addPath( model->sourcePath() );
   file.addPath( model->sourceFile() );
@@ -383,16 +465,19 @@ KUrl PatchReviewToolView::urlForFileModel(const Diff2::DiffModel* model)
 
 void PatchReviewToolView::kompareModelChanged()
 {
-    m_filesModel->clear();
-    m_filesModel->insertColumns( 0, 1 );
+    m_editPatch.filesList->clear();
+    m_editPatch.filesList->setColumnCount(1);
 
     if (!m_plugin->modelList())
         return;
 
+    QMap<KUrl, QString> additionalUrls = m_plugin->patch()->additionalSelectableFiles();
+    
     const Diff2::DiffModelList* models = m_plugin->modelList()->models();
     if ( !models )
         throw "no diff-models";
     Diff2::DiffModelList::const_iterator it = models->begin();
+    QSet<KUrl> haveUrls;
     while ( it != models->end() ) {
         Diff2::DifferenceList * diffs = ( *it ) ->differences();
         int cnt = 0;
@@ -401,17 +486,56 @@ void PatchReviewToolView::kompareModelChanged()
 
         KUrl file = urlForFileModel(*it);
 
-        m_filesModel->insertRow( 0 );
-        QModelIndex i = m_filesModel->index( 0, 0 );
-        if ( i.isValid() ) {
-            //m_filesModel->setData( i, file, Qt::DisplayRole );
-            m_filesModel->setData( i, QString( "%1 (%2 hunks)" ).arg( file.toLocalFile() ).arg( cnt ), Qt::DisplayRole );
-            QVariant v;
-            v.setValue<const Diff2::DiffModel*>( *it );
-            m_filesModel->setData( i, v, Qt::UserRole );
-        }
+        QTreeWidgetItem* item = new QTreeWidgetItem(m_editPatch.filesList);
+        
+        haveUrls.insert(file);
+        
+        m_editPatch.filesList->insertTopLevelItem(0, item);
+        QString text = QString( "%1 (%2 hunks" ).arg( ICore::self()->projectController()->prettyFileName(file, KDevelop::IProjectController::FormatPlain) ).arg( cnt );
+        
+        if(additionalUrls.contains(file) && !additionalUrls[file].isEmpty())
+          text += ", " + additionalUrls[file];
+        
+        text += ")";
+        
+        item->setData( 0, Qt::DisplayRole, text );
+        QVariant v;
+        v.setValue<const Diff2::DiffModel*>( *it );
+        item->setData( 0, Qt::UserRole, v );
+        item->setCheckState( 0, Qt::Checked );
         ++it;
     }
+    
+    ///First add files that a project was found for, then the other ones
+    ///findProject() excludes some useless files like backups, so we can use that to sort those files to the back
+    for(int withProject = 1; withProject >= 0; --withProject)
+    {
+      for(QMap<KUrl, QString>::const_iterator it = additionalUrls.begin(); it != additionalUrls.end(); ++it)
+      {
+        KUrl url = it.key();
+        
+          if(((bool)ICore::self()->projectController()->findProjectForUrl(url)) != (bool)withProject)
+            continue;
+        
+        if(!haveUrls.contains(url))
+        {
+          haveUrls.insert(url);
+          
+          QTreeWidgetItem* item = new QTreeWidgetItem(m_editPatch.filesList);
+          
+          m_editPatch.filesList->insertTopLevelItem(0, item);
+          QString text = ICore::self()->projectController()->prettyFileName(url, KDevelop::IProjectController::FormatPlain);
+          if(!(*it).isEmpty())
+            text += " (" + (*it) + ")";
+          
+          item->setData( 0, Qt::DisplayRole, text );
+          QVariant v;
+          v.setValue<KUrl>( url );
+          item->setData( 0, Qt::UserRole, v );
+          item->setCheckState( 0, Qt::Unchecked );
+        }
+      }
+    }
 }
 
 
@@ -420,27 +544,19 @@ void PatchReviewToolView::documentActivated(IDocument* doc)
     QModelIndexList i = m_editPatch.filesList->selectionModel() ->selectedIndexes();
     if ( !m_plugin->modelList() )
         return ;
-    for(uint a = 0; a < m_filesModel->rowCount(); ++a) {
+    for(uint a = 0; a < m_editPatch.filesList->topLevelItemCount(); ++a) {
       
-        QModelIndex index = m_filesModel->index(a, 0);
+        QTreeWidgetItem* item = m_editPatch.filesList->topLevelItem(a);
       
-        QVariant v = m_filesModel->data( index, Qt::UserRole );
+        QVariant v = item->data( 0, Qt::UserRole );
         if ( v.canConvert<const Diff2::DiffModel*>() ) {
             const Diff2::DiffModel * model = v.value<const Diff2::DiffModel*>();
             
             KUrl file = urlForFileModel(model);
             
             if(file == doc->url()) {
-              m_editPatch.filesList->setCurrentIndex(index);
+              m_editPatch.filesList->setCurrentItem(item);
               return;
-            
-              /*
-              if ( model ) {
-                  if ( model->differenceCount() != 0 ) {
-                      m_editPatch.nextHunk->setEnabled( true );
-                      m_editPatch.previousHunk->setEnabled( true );
-                  }
-              }*/
             }
         }
     }
@@ -492,9 +608,9 @@ void PatchHighlighter::showToolTipForMark(QPoint pos, KTextEditor::SmartRange* m
   
   if(diff->applied()) {
     if(m_plugin->isReverseChange())
-      html += i18n("<b>Reverted!</b><br/>");
+      html += i18n("<b>Reverted.</b><br/>");
     else
-      html += i18n("<b>Applied!</b><br/>");
+      html += i18n("<b>Applied.</b><br/>");
     
     html += i18n("<b>Previous:</b><br/>");
     lines = diff->sourceLines();
@@ -609,7 +725,7 @@ void PatchHighlighter::markClicked(KTextEditor::Document* doc, KTextEditor::Mark
     }
     
     if(currentText.join("\n").simplified() != replace.simplified()) {
-      KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("Could not apply the change: Text should be \"%1\", but is \"%2\"", replace, currentText.join("\n")));
+      KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("Could not apply the change: Text should be \"%1\", but is \"%2\".", replace, currentText.join("\n")));
       return;
     }
     
@@ -904,51 +1020,20 @@ void PatchReviewPlugin::removeHighlighting( const KUrl& file ) {
 
 void PatchReviewPlugin::notifyPatchChanged()
 {
-    kDebug() << "notifying patch change: " << m_patch->filename;
+    kDebug() << "notifying patch change: " << m_patch->file();
     m_updateKompareTimer->start(500);
-    m_diffFile = KUrl();
 }
 
 void PatchReviewPlugin::showPatch()
 {
-//     KRun::displayOpenWithDialog(KUrl::List() << diffFile(), core()->uiController()->activeMainWindow());
-      KRun::runCommand("kompare " + diffFile().pathOrUrl(), core()->uiController()->activeMainWindow());
+    startReview(m_patch, OpenAndRaise);
 }
 
 void PatchReviewPlugin::forceUpdate()
 {
+  m_patch->update();
+  
   notifyPatchChanged();
-
-  if(!m_patch->command.isEmpty()) {
-    KTemporaryFile temp;
-    temp.setSuffix(".diff");
-    temp.setAutoRemove(false);
-    if(temp.open()) {
-      temp.setAutoRemove(false);
-      QString filename = temp.fileName();
-      kDebug() << "temp file: " << filename;
-      temp.close();
-      KProcess proc;
-      proc.setWorkingDirectory(m_patch->baseDir.toLocalFile());
-      proc.setOutputChannelMode(KProcess::OnlyStdoutChannel);
-      proc.setStandardOutputFile(filename);
-      ///Try to apply, if it works, the patch is not applied
-      proc << splitArgs( m_patch->command );
-
-      kDebug() << "calling " << m_patch->command;
-
-      if ( proc.execute() ) {
-          kWarning() << "returned with bad exit code";
-          return;
-      }
-      
-      m_diffFile = KUrl(filename);
-      kDebug() << "success, diff: " << m_diffFile;
-      
-    }else{
-      kWarning() << "PROBLEM";
-    }
-  }
 }
 
 void PatchReviewPlugin::updateKompareModel() {
@@ -970,7 +1055,7 @@ void PatchReviewPlugin::updateKompareModel() {
         try {
             ///@todo does not work with remote URLs
             ///@todo Only reverse if the patch is currently applied
-            if ( !m_modelList->openDirAndDiff( m_patch->baseDir.toLocalFile(), diffFile.toLocalFile(), true ) )
+            if ( !m_modelList->openDirAndDiff( m_patch->baseDir().toLocalFile(), diffFile.toLocalFile(), true ) )
                 throw "could not open diff " + diffFile.toLocalFile();
          } catch ( const QString & str ) {
             throw;
@@ -1015,7 +1100,7 @@ public:
 
     virtual QString id() const
     {
-        return "org.kdevelop.PatchReviewToolview";
+        return "org.kdevelop.PatchReview";
     }
 
 private:
@@ -1025,20 +1110,182 @@ private:
 PatchReviewPlugin::~PatchReviewPlugin()
 {
     removeHighlighting();
+    delete m_patch;
 }
 
-void PatchReviewPlugin::commandToFile()
+void PatchReviewPlugin::registerPatch(IPatchSource::Ptr patch)
 {
-    if(!diffFile().isEmpty()) {
-      m_patch->filename = diffFile();
-      m_patch->command.clear();
-      notifyPatchChanged();
+  if(!m_knownPatches.contains(patch)) {
+    m_knownPatches << patch;
+    connect(patch, SIGNAL(destroyed(QObject*)), SLOT(clearPatch(QObject*)));
+  }
+}
+
+void PatchReviewPlugin::clearPatch(QObject* _patch)
+{
+  kDebug() << "clearing patch" << _patch << "current:" << (QObject*)m_patch;
+  IPatchSource::Ptr patch((IPatchSource*)_patch);
+  m_knownPatches.removeAll(patch);
+  m_knownPatches.removeAll(0);
+  
+  if(patch == m_patch) {
+    kDebug() << "is current patch";
+    if(!m_knownPatches.empty())
+      setPatch(m_knownPatches.first());
+    else
+      setPatch(IPatchSource::Ptr(new LocalPatchSource));
+  }
+}
+
+#if 0
+#if HAVE_KOMPARE
+void showDiff(const KDevelop::VcsDiff& d)
+{
+    ICore::self()->uiController()->switchToArea("review", KDevelop::IUiController::ThisWindow);
+    foreach(const VcsLocation& l, d.leftTexts().keys())
+    {
+        KUrl to;
+        if(d.rightTexts().contains(l))
+        {
+            KTemporaryFile temp2;
+            temp2.setSuffix("2.patch");
+            temp2.setAutoRemove(false);
+            temp2.open();
+            QTextStream t2(&temp2);
+            t2 << d.rightTexts()[l];
+            temp2.close();
+            to=temp2.fileName();
+        }
+        else
+            to=l.localUrl();
+        
+        KUrl fakeUrl(to);
+        fakeUrl.setScheme("kdevpatch");
+        
+        IDocumentFactory* docf=ICore::self()->documentController()->factory("text/x-patch");
+        IDocument* doc=docf->create(fakeUrl, ICore::self());
+        IPatchDocument* pdoc=dynamic_cast<IPatchDocument*>(doc);
+        
+        Q_ASSERT(pdoc);
+        ICore::self()->documentController()->openDocument(doc);
+        pdoc->setDiff(d.leftTexts()[l], to);
     }
 }
+#endif
+#endif
 
-PatchReviewPlugin::PatchReviewPlugin(QObject *parent, const QVariantList &) : KDevelop::IPlugin(KDevProblemReporterFactory::componentData(), parent), m_factory(new PatchReviewToolViewFactory(this)) {
+void PatchReviewPlugin::cancelReview()
+{
+  if(m_patch) {
+    m_modelList.reset( 0 );
+    m_patch->cancelReview();
+    
+    delete m_patch;
+    
+    Sublime::MainWindow* w = dynamic_cast<Sublime::MainWindow*>(ICore::self()->uiController()->activeMainWindow());
+    if(w->area()->workingSet().startsWith("review")) {
+      w->area()->clearViews();
+      ICore::self()->uiController()->switchToArea("code", KDevelop::IUiController::ThisWindow);
+    }
+  }
+}
 
-    m_patch = LocalPatchSourcePointer(new LocalPatchSource);
+void PatchReviewPlugin::finishReview(QList< KUrl > selection)
+{
+  if(m_patch) {
+    if(!m_patch->finishReview(selection))
+      return;
+    m_modelList.reset( 0 );
+    if(!dynamic_cast<LocalPatchSource*>(m_patch))
+      delete m_patch;
+    
+    Sublime::MainWindow* w = dynamic_cast<Sublime::MainWindow*>(ICore::self()->uiController()->activeMainWindow());
+    if(w->area()->workingSet().startsWith("review")) {
+      w->area()->clearViews();
+      ICore::self()->uiController()->switchToArea("code", KDevelop::IUiController::ThisWindow);
+    }
+  }
+}
+
+void PatchReviewPlugin::startReview(IPatchSource* patch, IPatchReview::ReviewMode mode)
+{
+  setPatch(patch);
+  QMetaObject::invokeMethod(this, "updateReview", Qt::QueuedConnection);
+}
+
+void PatchReviewPlugin::updateReview()
+{
+  if(!m_patch)
+    return;
+  
+  m_updateKompareTimer->stop();
+  updateKompareModel();
+  ICore::self()->uiController()->switchToArea("review", KDevelop::IUiController::ThisWindow);
+  Sublime::MainWindow* w = dynamic_cast<Sublime::MainWindow*>(ICore::self()->uiController()->activeMainWindow());
+  if(!w->area()->workingSet().startsWith("review"))
+    w->area()->setWorkingSet("review");
+  
+  w->area()->clearViews();
+  
+  if(!m_modelList.get())
+    return;
+  
+  //Open the diff itself
+#if HAVE_KOMPARE
+  KUrl fakeUrl(m_patch->file());
+  fakeUrl.setScheme("kdevpatch");
+  IDocumentFactory* docf=ICore::self()->documentController()->factory("text/x-patch");
+  IDocument* doc=docf->create(fakeUrl, ICore::self());
+  IPatchDocument* pdoc=dynamic_cast<IPatchDocument*>(doc);
+  
+  Q_ASSERT(pdoc);
+  ICore::self()->documentController()->openDocument(doc);
+#else
+  ICore::self()->documentController()->openDocument(m_patch->file());
+#endif
+
+  if(m_modelList->modelCount() < 10) {
+    //Open all relates files
+    for(int a = 0; a < m_modelList->modelCount(); ++a) {
+      KUrl url(m_modelList->modelAt(a)->source());
+       ICore::self()->documentController()->openDocument(url);
+       seekHunk(true, url); //Jump to the first changed position
+    }
+  }
+  
+  bool b = ICore::self()->uiController()->findToolView(i18n("Patch Review"), m_factory);
+  Q_ASSERT(b);
+}
+
+void PatchReviewPlugin::setPatch(IPatchSource* patch)
+{
+  if(m_patch) {
+    QObject* objPatch = dynamic_cast<QObject*>(m_patch);
+    if(objPatch) {
+      disconnect(objPatch, SIGNAL(patchChanged()), this, SLOT(notifyPatchChanged()));
+    }
+  }
+  m_patch = patch;
+
+  if(m_patch) {
+    kDebug() << "setting new patch" << patch->name() << "with file" << patch->file();
+    registerPatch(patch);
+    
+    QObject* objPatch = dynamic_cast<QObject*>(m_patch);
+    if(objPatch) {
+      connect(objPatch, SIGNAL(patchChanged()), this, SLOT(notifyPatchChanged()));
+    }
+  }
+  
+  notifyPatchChanged();
+}
+
+PatchReviewPlugin::PatchReviewPlugin(QObject *parent, const QVariantList &)
+: KDevelop::IPlugin(KDevProblemReporterFactory::componentData(), parent), 
+  m_patch(0), m_factory(new PatchReviewToolViewFactory(this))
+{
+    KDEV_USE_EXTENSION_INTERFACE( KDevelop::IPatchReview )
+  
     core()->uiController()->addToolView(i18n("Patch Review"), m_factory);
     setXMLFile("kdevpatchreview.rc");
 
@@ -1048,6 +1295,8 @@ PatchReviewPlugin::PatchReviewPlugin(QObject *parent, const QVariantList &) : KD
     m_updateKompareTimer = new QTimer( this );
     m_updateKompareTimer->setSingleShot( true );
     connect( m_updateKompareTimer, SIGNAL( timeout() ), this, SLOT( updateKompareModel() ) );
+    
+    setPatch(IPatchSource::Ptr(new LocalPatchSource));
 }
 
 void PatchReviewPlugin::documentClosed(IDocument* doc)
