@@ -11,8 +11,8 @@ Copyright 2006 David Nolden <david.nolden.kdevelop@art-master.de>
  *                                                                         *
  ***************************************************************************/
 
-#ifndef EDITPATCH_H
-#define EDITPATCH_H
+#ifndef PATCHREVIEW_H
+#define PATCHREVIEW_H
 
 #include <QObject>
 #include <QPointer>
@@ -58,6 +58,9 @@ public:
     PatchHighlighter( const Diff2::DiffModel* model, KDevelop::IDocument* doc, PatchReviewPlugin* plugin ) throw( QString );
     ~PatchHighlighter();
     KDevelop::IDocument* doc();
+    QList<KTextEditor::SmartRange*> ranges() const {
+      return m_differencesForRanges.keys();
+    }
 private slots:
     void documentDestroyed();
 private:
@@ -107,11 +110,12 @@ private slots:
 
     void slotEditFileNameChanged();
 
+    void finishReview();
+    
 private:
     void kompareModelChanged();
 
     void showEditDialog();
-    void editPatchReadOnly();
     ///Fills the editor views from m_editingPatch
     void fillEditFromPatch();
 
@@ -123,17 +127,21 @@ private:
     QPointer<KParts::Part> m_konsolePart;
 
     bool m_reversed;
-    QStandardItemModel* m_filesModel;
 
     KUrl urlForFileModel(const Diff2::DiffModel* model);
     
     PatchReviewPlugin* m_plugin;
+    
+    QPointer<QWidget> m_customWidget;
 public slots:
     void documentActivated(KDevelop::IDocument*);
+    void patchSelectionChanged(int);
 };
 
-class PatchReviewPlugin : public KDevelop::IPlugin {
+class PatchReviewPlugin : public KDevelop::IPlugin, public KDevelop::IPatchReview {
     Q_OBJECT
+    Q_INTERFACES( KDevelop::IPatchReview )
+    
 public:
     PatchReviewPlugin(QObject *parent, const QVariantList & = QVariantList() );
     ~PatchReviewPlugin();
@@ -141,16 +149,18 @@ public:
 
     QWidget* createToolView(QWidget* parent);
 
-    LocalPatchSourcePointer patch() const {
+    KDevelop::IPatchSource::Ptr patch() const {
         return m_patch;
     }
 
+    QList<KDevelop::IPatchSource::Ptr> knownPatches() const {
+      return m_knownPatches;
+    }
+
     Diff2::KompareModelList* modelList() const {
         return m_modelList.get();
     }
 
-    void notifyPatchChanged();
-
     void seekHunk( bool forwards, const KUrl& file = KUrl() );
 
     KUrl diffFile();
@@ -158,26 +168,41 @@ public:
     ///Returns whether the change is reversed:
     ///The source source is the current version of the document, and the destination is the version of the document before the patch was applied.
     bool isReverseChange() const ;
+
+    void setPatch(KDevelop::IPatchSource* patch);
+
+    void registerPatch(KDevelop::IPatchSource::Ptr patch);
+    
+    virtual void startReview(KDevelop::IPatchSource* patch, ReviewMode mode);
+    
+    void finishReview(QList<KUrl> selection);
     
 Q_SIGNALS:
     void patchChanged();
 
 public Q_SLOTS:
-    void  highlightPatch();
+    //Does parts of the review-starting that are problematic to do directly in startReview, as they may open dialogs etc.
+    void updateReview();
+    
+    void cancelReview();
+    void clearPatch(QObject* patch);
+    void notifyPatchChanged();
+    void highlightPatch();
     void updateKompareModel();
     void showPatch();
-    void commandToFile();
     void forceUpdate();
 
 private Q_SLOTS:
     void documentClosed(KDevelop::IDocument*);
     void textDocumentCreated(KDevelop::IDocument*);
 private:
+  
+    QList<KDevelop::IPatchSource::Ptr> m_knownPatches;
 
     void addHighlighting( const KUrl& file, KDevelop::IDocument* document = 0 );
     void removeHighlighting( const KUrl& file = KUrl() );
 
-    LocalPatchSourcePointer m_patch;
+    KDevelop::IPatchSource* m_patch;
 
     QTimer* m_updateKompareTimer;
 
