@@ -28,6 +28,7 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kio/netaccess.h>
+#include <kio/copyjob.h>
 #include <kparts/mainwindow.h>
 #include <kparts/componentfactory.h>
 #include <KTemporaryFile>
@@ -127,19 +128,22 @@ ProjectManagerViewPlugin::ProjectManagerViewPlugin( QObject *parent, const QVari
     connect( d->m_build, SIGNAL(triggered()), this, SLOT(buildProjectItems()) );
     actionCollection()->addAction( "project_build", d->m_build );
     d->m_install = new KAction( i18n("Install"), this );
+    d->m_install->setIcon(KIcon("run-build-install"));
     d->m_install->setEnabled( false );
     connect( d->m_install, SIGNAL(triggered()), this, SLOT(installProjectItems()) );
     actionCollection()->addAction( "project_install", d->m_install );
     d->m_clean = new KAction( i18n("Clean"), this );
+    d->m_clean->setIcon(KIcon("run-build-clean"));
     d->m_clean->setEnabled( false );
     connect( d->m_clean, SIGNAL(triggered()), this, SLOT(cleanProjectItems()) );
     actionCollection()->addAction( "project_clean", d->m_clean );
     d->m_configure = new KAction( i18n("Configure"), this );
-    d->m_configure->setIcon(KIcon("configure"));
+    d->m_configure->setIcon(KIcon("run-build-configure"));
     d->m_configure->setEnabled( false );
     connect( d->m_configure, SIGNAL(triggered()), this, SLOT(configureProjectItems()) );
     actionCollection()->addAction( "project_configure", d->m_configure );
     d->m_prune = new KAction( i18n("Prune"), this );
+    d->m_prune->setIcon(KIcon("run-build-prune"));
     d->m_prune->setEnabled( false );
     connect( d->m_prune, SIGNAL(triggered()), this, SLOT(pruneProjectItems()) );
     actionCollection()->addAction( "project_prune", d->m_prune );
@@ -221,6 +225,7 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
     bool fileItemsAdded = false;
     bool folderWithParentAdded = false;
     bool targetAdded = false;
+    bool renameAdded = false;
     foreach( ProjectBaseItem* item, items )
     {
         d->ctxProjectItemList << item;
@@ -299,6 +304,15 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
             menuExt.addAction( ContextMenuExtension::FileGroup, action );
         }
         
+        if( !renameAdded && (item->file() || item->folder()) )
+        {
+            renameAdded = true;
+            KAction* action = new KAction( i18n( "Rename" ), this );
+            action->setIcon(KIcon("edit-rename"));
+            connect( action, SIGNAL(triggered()), this, SLOT(renameItemFromContextMenu()) );
+            menuExt.addAction( ContextMenuExtension::FileGroup, action );
+        }
+        
         //TODO: Port to launch framework
 //         if(!hasTargets && item->executable())
 //         {
@@ -518,6 +532,48 @@ void ProjectManagerViewPlugin::removeFileFromContextMenu()
     }
 }
 
+void ProjectManagerViewPlugin::renameItemFromContextMenu()
+{
+    QWidget* window=ICore::self()->uiController()->activeMainWindow()->window();
+    
+    foreach( KDevelop::ProjectBaseItem* item, d->ctxProjectItemList )
+    {
+        KDevelop::ProjectFileItem* file=item->file();
+        KDevelop::ProjectFolderItem* folder=item->folder();
+        
+        if(file) {
+            //Change QInputDialog->KFileSaveDialog?
+            QString name = QInputDialog::getText( window, i18n("Rename File"), i18n("New name for '%1'", item->text()) );
+            if (!name.isEmpty()) {
+                KUrl url = file->url().upUrl();
+                url.addPath( name );
+                
+                KIO::CopyJob* job=KIO::move(file->url(), url);
+                if(!KIO::NetAccess::synchronousRun(job, window)) {
+                    KMessageBox::error( window, i18n("Cannot rename '%1'.", file->url().prettyUrl()) );
+                    continue;
+                }
+                item->project()->projectFileManager()->renameFile(file, url);
+            }
+        } else if(folder) {
+            //Change QInputDialog->KFileSaveDialog?
+            QString name = QInputDialog::getText( window, i18n("Rename Folder"), i18n("New name for '%1'", item->text()) );
+            if (!name.isEmpty()) {
+                KUrl url = folder->url().upUrl();
+                url.addPath( name );
+                
+                KIO::CopyJob* job=KIO::move(folder->url(), url);
+                if(!KIO::NetAccess::synchronousRun(job, window)) {
+                    KMessageBox::error( window, i18n("Cannot rename '%1'.", folder->url().prettyUrl()) );
+                    continue;
+                }
+                item->project()->projectFileManager()->renameFolder(folder, url);
+            }
+            
+        }
+    }
+}
+
 ProjectFileItem* createFile(const ProjectFolderItem* item)
 {
     QWidget* window = ICore::self()->uiController()->activeMainWindow()->window();
