@@ -73,6 +73,12 @@ public:
     //Frees the model by closing active quickopen dialoags, and retuns whether successful.
     bool freeModel();
     
+    virtual void createActionsForMainWindow( Sublime::MainWindow* window, QString& xmlFile, KActionCollection& actions );
+
+    QuickOpenLineEdit* createQuickOpenLineWidget();
+    
+    virtual KDevelop::IQuickOpenLine* createQuickOpenLine(const QStringList& scopes, const QStringList& type, QuickOpenType kind);
+    
 public slots:
     void quickOpen();
     void quickOpenFile();
@@ -83,14 +89,20 @@ public slots:
     void quickOpenNavigate();
     void quickOpenNavigateFunctions();
 
+    void previousFunction();
+    void nextFunction();
+
 private slots:
     void storeScopes( const QStringList& );
     void storeItems( const QStringList& );
 
 private:
     friend class QuickOpenLineEdit;
-    QuickOpenLineEdit* quickOpenLine();
-    QWidget* createQuickOpenLineWidget();
+    friend class StandardQuickOpenWidgetCreator;
+    QuickOpenLineEdit* quickOpenLine(QString name = "Quickopen");
+
+    enum FunctionJumpDirection { NextFunction, PreviousFunction };
+    void jumpToNearestFunction(FunctionJumpDirection direction);
 
     QPair<KUrl, KDevelop::SimpleCursor> specialObjectJumpPosition() const;
     QWidget* specialObjectNavigationWidget() const;
@@ -122,7 +134,7 @@ class QuickOpenWidget : public QFrame {
   void setPreselectedText(const QString &text);
     void prepareShow();
 
-  void setAlternativeSearchField(QLineEdit* alterantiveSearchField);
+  void setAlternativeSearchField(KLineEdit* alterantiveSearchField);
     
   //Shows OK + Cancel. By default they are hidden  
   void showStandardButtons(bool show);
@@ -151,6 +163,7 @@ class QuickOpenWidget : public QFrame {
   bool m_expandedTemporary, m_hadNoCommandSinceAlt;
   QTime m_altDownTime;
   QString m_preselectedText;
+  public:
   Ui::QuickOpen o;
   
   friend class QuickOpenWidgetDialog;
@@ -173,26 +186,34 @@ class QuickOpenWidgetDialog : public QObject {
   QuickOpenWidget* m_widget;
 };
 
-class QuickOpenLineEdit : public QLineEdit {
+class QuickOpenWidgetCreator;
+
+class QuickOpenLineEdit : public KDevelop::IQuickOpenLine {
   Q_OBJECT
   public:
-    QuickOpenLineEdit() ;
+    QuickOpenLineEdit(QuickOpenWidgetCreator* creator) ;
     ~QuickOpenLineEdit() ;
     
     bool insideThis(QObject* object);
     void showWithWidget(QuickOpenWidget* widget);
+    
+    virtual void setDefaultText(QString text) {
+      m_defaultText = text;
+      setClickMessage(m_defaultText);
+    }
   private slots:
     void activate() ;
     void deactivate() ;
     void checkFocus();
   private:
-    virtual void keyPressEvent(QKeyEvent* ev) ;
     virtual void focusInEvent(QFocusEvent* ev) ;
     virtual bool eventFilter(QObject* obj, QEvent* e) ;
     virtual void hideEvent(QHideEvent* );
     
     QPointer<QuickOpenWidget> m_widget;
     bool m_forceUpdate;
+    QString m_defaultText;
+    QuickOpenWidgetCreator* m_widgetCreator;
 };
 
 #endif // QUICKOPENPLUGIN_H
