@@ -16,10 +16,13 @@
 #include <QFile>
 #include <QTextStream>
 #include <QDir>
+#include <QApplication>
+
+#include <KMessageBox>
+#include <KLocalizedString>
 
 #include "snippetstore.h"
 #include "snippetrepository.h"
-#include "snippetvariablesubst.h"
 
 Snippet::Snippet(const QString& filename, SnippetRepository* repo)
     : QStandardItem(filename), repo_(repo), name_(filename)
@@ -55,7 +58,7 @@ void Snippet::save()
         input << snippetText_;
 
         foreach(const QString& keyword, keywords_) {
-            input << SNIPPET_METADATA << "keyword="<<keyword<<endl;
+            input << endl << SNIPPET_METADATA << "keyword=" << keyword;
         }
     }
 }
@@ -83,9 +86,13 @@ void Snippet::changeName(const QString& newName)
         QString newFileName = getFileName();
 
         if ( QFile::exists( origFileName ) ) {
-            bool ok = QFile::rename(origFileName, newFileName);
-            if (ok) {
+            if (QFile::rename(origFileName, newFileName)) {
                 setText( newName );
+            } else {
+                KMessageBox::error(
+                    QApplication::activeWindow(),
+                    i18n("Could not move snippet file from \"%1\" to \"%2\".", origFileName, newFileName)
+                );
             }
         } else {
             setText( newName );
@@ -98,7 +105,7 @@ void Snippet::changeName(const QString& newName)
 
 void Snippet::setRawData(const QString& data)
 {
-    QStringList rows = data.split( QRegExp("[\\r\\n]+") );
+    QStringList rows = data.split( QRegExp("[\\r\\n]+"), QString::SkipEmptyParts );
     QStringList metadata;
 
     QString newText;
@@ -106,16 +113,18 @@ void Snippet::setRawData(const QString& data)
     // A snippet file can contain meta information
     // which needs to be separated from the snippet's text
     QStringListIterator it(rows);
+    bool needNewline = false;
     while (it.hasNext()) {
         QString str = it.next();
 
         if (str.startsWith(SNIPPET_METADATA)) {
             metadata << str;
         } else {
-            newText += str;
-            if (it.hasNext()) {
+            if ( needNewline ) {
                 newText += SNIPPET_END_OF_LINE;
             }
+            newText += str;
+            needNewline = true;
         }
     }
 
@@ -154,69 +163,16 @@ void Snippet::handleMetaData(QStringList& metadata)
     }
 }
 
-int Snippet::initVariableModel(SnippetVariableModel& model)
-{
-    // The set helps us to ensure that each variable gets found only once
-    QSet<QString> set;
-
-    // This regexp finds all variables in the snippet
-    QRegExp rx("\\$(\\w+)\\$");
-    int pos = 0;
-    while ((pos = rx.indexIn(snippetText_, pos)) != -1) {
-        set << rx.cap(1);
-        pos += rx.matchedLength();
-    }
-
-    // Copy the variable names into the given model
-    int count = 0;
-    foreach (const QString& str, set) {
-        model.appendVariable( str );
-        count++;
-    }
-
-    return count;
-}
-
-const QString Snippet::interpretSnippet()
-{
-    SnippetVariableModel variables;
-
-    // First we need to check if there are variables in the snippet
-    // If so, they get copied into the model
-    int count = initVariableModel( variables );
-
-    QString snippet = snippetText_;
-
-    // first we need to replace the double-delimiter into a single delimiter
-    snippet.replace( "$$", "$" );
-
-    if (count > 0) {
-        // open the SnippetVariables dialog and give it our model
-        SnippetVariables dlg( &variables );
-        if ( dlg.exec() == QDialog::Accepted ) {
-            // If the user accepted the dialog, it holds a list of variables and the
-            // values the user wants to substitute
-
-            SnippetVariableItem* variable;
-            foreach(variable, variables.getVariableList()) {
-                snippet.replace( '$'+variable->name()+'$', variable->value() );
-            }
-        } else {
-            // the user canceled the dialog; return an empty string
-            return QString();
-        }
-    }
-
-    // In case the snippet does not contain variables, just return the text
-    return snippet;
-}
-
 void Snippet::removeSnippetFile()
 {
-    bool ok = QFile::remove( getFileName() );
-    if (ok) {
+    if (!QFile::exists( getFileName() ) || QFile::remove( getFileName() )) {
         // if the file has been removed, also remove the Snippet from the model
         QStandardItem::parent()->removeRows( row(), 1 );
+    } else {
+        KMessageBox::error(
+            QApplication::activeWindow(),
+            i18n("Could not remove snippet file \"%1\".", getFileName())
+        );
     }
 }
 
