@@ -21,6 +21,10 @@
 #include "snippetstore.h"
 #include "snippet.h"
 
+#include <KMessageBox>
+#include <KLocalizedString>
+#include <QApplication>
+
 SnippetRepository::SnippetRepository(const QString& name, const QString& location)
  : QStandardItem(name)
 {
@@ -35,14 +39,6 @@ SnippetRepository::~SnippetRepository()
 {
     // remove all our children from both the model and our internal data structures
     removeRows( 0, rowCount() );
-
-    if (index().isValid()) {
-        // in case this item is still a member of a model, remove itself
-        QStandardItem::parent()->removeRows( row(), 1 );
-    }
-
-    snippets_.clear();
-    subrepos_.clear();
 }
 
 void SnippetRepository::slotSyncRepository()
@@ -50,8 +46,6 @@ void SnippetRepository::slotSyncRepository()
     // as we are going to reget all snippets from this base directory
     // we first need to remove all snippets and sub-repos
     removeRows( 0, rowCount() );
-    snippets_.clear();
-    subrepos_.clear();
 
     // now reget all snippets and sub-repos
     QFSFileEngine engine( location_ );
@@ -87,10 +81,38 @@ void SnippetRepository::slotSyncRepository()
     }
 }
 
+const QList< Snippet* > SnippetRepository::getSnippets() const
+{
+    QList< Snippet* > snippets;
+    for ( int i = 0; i < rowCount(); ++i ) {
+        if ( Snippet* snippet = dynamic_cast<Snippet*>(child(i)) ) {
+            snippets << snippet;
+        } else if ( SnippetRepository* repo = dynamic_cast<SnippetRepository*>(child(i)) ) {
+            snippets += repo->getSnippets();
+        }
+    }
+    return snippets;
+}
+
+const QString& SnippetRepository::getLocation() const
+{
+    return location_;
+}
+
+void SnippetRepository::addSubRepo(SnippetRepository* repo)
+{
+    appendRow( repo );
+}
+
 void SnippetRepository::addSnippet( Snippet* snippet )
 {
     appendRow( snippet );
-    snippets_.append( snippet );
+}
+
+void SnippetRepository::setLocation(QString loc)
+{
+    location_ = QDir::cleanPath(loc);
+    setToolTip(location_);
 }
 
 void SnippetRepository::changeLocation(const QString& newLocation, const QString& newName)
@@ -105,8 +127,7 @@ void SnippetRepository::changeLocation(const QString& newLocation, const QString
     //if the location changed, try to move the repository
     QString oldLocation = getLocation();
     QDir dir(oldLocation);
-    bool ok = dir.rename( oldLocation, newLocation );
-    if (ok) {
+    if (dir.rename( oldLocation, newLocation )) {
         // if moving was ok, set the location, tooltip and the name
         setLocation( newLocation );
         if (!newName.isEmpty())
@@ -115,15 +136,35 @@ void SnippetRepository::changeLocation(const QString& newLocation, const QString
         // as the snippets and subrepos now changed their
         // location on the disk, resync everything in this repo
         slotSyncRepository();
+    } else {
+        KMessageBox::error(
+            QApplication::activeWindow(),
+            i18n("Could not move repository from \"%1\" to \"%2\".", oldLocation, newLocation)
+        );
     }
 }
 
 void SnippetRepository::removeDirectory()
 {
     QDir dir( getLocation() );
-    bool ok = dir.rmdir( getLocation() );
-    if (ok) {
-        QStandardItem::parent()->removeRows( row(), 1 );
+    // remove all contents
+    for ( int i = 0; i < rowCount(); ++i ) {
+        if ( Snippet* snippet = dynamic_cast<Snippet*>(child(i)) ) {
+            snippet->removeSnippetFile();
+        } else if ( SnippetRepository* repo = dynamic_cast<SnippetRepository*>(child(i)) ) {
+            repo->removeDirectory();
+        }
+    }
+    if (!dir.exists() || dir.rmdir( getLocation() )) {
+        if (index().isValid()) {
+            // in case this item is still a member of a model, remove itself
+            QStandardItem::parent()->removeRows( row(), 1 );
+        }
+    } else {
+        KMessageBox::error(
+            QApplication::activeWindow(),
+            i18n("Could not remove repository \"%1\".", getLocation())
+        );
     }
 }
 
