@@ -99,7 +99,7 @@ KDevSvnPlugin::~KDevSvnPlugin()
 
 bool KDevSvnPlugin::isVersionControlled(const KUrl& localLocation)
 {
-    SvnInfoJob* job = new SvnInfoJob(this, KDevelop::OutputJob::Silent);
+    SvnInfoJob* job = new SvnInfoJob(this);
 
     job->setLocation(localLocation);
 
@@ -319,7 +319,7 @@ KDevelop::ContextMenuExtension KDevSvnPlugin::contextMenuExtension(KDevelop::Con
 
     bool hasVersionControlledEntries = false;
     foreach(const KUrl &url, ctxUrlList) {
-        if (isVersionControlled(url)) {
+        if (isVersionControlled(url) || isVersionControlled(url.upUrl())) {
             hasVersionControlledEntries = true;
             break;
         }
