@@ -36,6 +36,7 @@ class RunController;
 class SessionController;
 class CorePrivate;
 class SourceFormatterController;
+class ProgressManager;
 class SelectionController;
 class DocumentationController;
 class DebugController;
@@ -70,7 +71,7 @@ public:
     virtual ILanguageController *languageController();
 
     /** @copydoc ICore::partController() */
-    virtual KParts::PartManager *partController();
+    virtual IPartController *partController();
 
     /** @copydoc ICore::documentController() */
     virtual IDocumentController *documentController();
@@ -129,6 +130,11 @@ public:
      */
     SessionController *sessionController();
 
+    /**
+     * @internal
+     */
+    ProgressManager *progressController();
+
     void cleanup();
 
     Core::Setup setupFlags() const;
