@@ -99,8 +99,7 @@ private:
 DebugController::DebugController(QObject *parent)
     : IDebugController(parent), KXMLGUIClient(),
       m_breakpointModel(new BreakpointModel(this)),
-      m_variableCollection(new VariableCollection(this)),
-      m_frameStackModel(new FrameStackModel(this))
+      m_variableCollection(new VariableCollection(this))
 {
     setComponentData(KComponentData("kdevdebugger"));
     setXMLFile("kdevdebuggershellui.rc");
@@ -140,6 +139,16 @@ void DebugController::initialize()
     stateChanged("ended");
 }
 
+
+void DebugController::cleanup()
+{
+    if (m_currentSession) m_currentSession->stopDebugger();
+}
+
+DebugController::~DebugController()
+{
+}
+
 BreakpointModel* DebugController::breakpointModel()
 {
     return m_breakpointModel;
@@ -150,13 +159,6 @@ VariableCollection* DebugController::variableCollection()
     return m_variableCollection;
 }
 
-FrameStackModel* DebugController::frameStackModel()
-{
-    return m_frameStackModel;
-}
-
-
-
 void DebugController::partAdded(KParts::Part* part)
 {
     if (KTextEditor::Document* doc = dynamic_cast<KTextEditor::Document*>(part)) {
@@ -280,21 +282,24 @@ void DebugController::setupActions()
 void DebugController::addSession(IDebugSession* session)
 {
     kDebug() << session;
+    Q_ASSERT(session->variableController());
+    Q_ASSERT(session->breakpointController());
+    Q_ASSERT(session->frameStackModel());
 
     //TODO support multiple sessions
     if (m_currentSession) {
         m_currentSession->stopDebugger();
     }
     m_currentSession = session;
-    
+        
     connect(session, SIGNAL(stateChanged(KDevelop::IDebugSession::DebuggerState)), SLOT(debuggerStateChanged(KDevelop::IDebugSession::DebuggerState)));
     connect(session, SIGNAL(showStepInSource(KUrl,int)), SLOT(showStepInSource(KUrl,int)));
     connect(session, SIGNAL(clearExecutionPoint()), SLOT(clearExecutionPoint()));
     connect(session, SIGNAL(raiseFramestackViews()), SIGNAL(raiseFramestackViews()));
     
     updateDebuggerState(session->state(), session);
-    
-    emit sessionAdded(session);
+        
+    emit currentSessionChanged(session);
     
     if((Core::self()->setupFlags() & Core::NoUi)) return;
 
@@ -325,9 +330,12 @@ void DebugController::showStepInSource(const KUrl &url, int lineNum)
 
     clearExecutionPoint();
     kDebug() << url << lineNum;
+
+    Q_ASSERT(dynamic_cast<IDebugSession*>(sender()));
+    KUrl openUrl = static_cast<IDebugSession*>(sender())->convertToLocalUrl(url);
     KDevelop::IDocument* document = KDevelop::ICore::self()
         ->documentController()
-        ->openDocument(url, KTextEditor::Cursor(lineNum, 0));
+        ->openDocument(openUrl, KTextEditor::Cursor(lineNum, 0));
 
     if( !document )
         return;
@@ -352,10 +360,11 @@ void DebugController::debuggerStateChanged(KDevelop::IDebugSession::DebuggerStat
     }
 
     if (state == IDebugSession::EndedState) {
-        session->deleteLater();
         if (session == m_currentSession) {
             m_currentSession = 0;
+            emit currentSessionChanged(0);
         }
+        session->deleteLater();
     }
 }
 
