@@ -43,7 +43,9 @@ class DebugController : public IDebugController, public KXMLGUIClient
     Q_OBJECT
 public:
     DebugController(QObject *parent = 0);
+    ~DebugController();
     void initialize();
+    void cleanup();
 
     virtual void addSession(IDebugSession* session);
     virtual IDebugSession* currentSession();
@@ -52,7 +54,6 @@ public:
 
     virtual BreakpointModel* breakpointModel();
     virtual VariableCollection* variableCollection();
-    virtual FrameStackModel *frameStackModel();
 
 private Q_SLOTS:
     void restartDebugger();
@@ -98,8 +99,7 @@ private:
 
     QPointer<IDebugSession> m_currentSession;
     BreakpointModel *m_breakpointModel;
-    VariableCollection *m_variableCollection;
-    FrameStackModel *m_frameStackModel;
+    VariableCollection *m_variableCollection;    
 };
 
 }
