@@ -23,24 +23,37 @@
 #include <KToolBar>
 #include <interfaces/idocumentation.h>
 
+class QModelIndex;
+class KLineEdit;
+class ProvidersModel;
+class QComboBox;
+
 class DocumentationView : public QWidget
 {
-	Q_OBJECT
-	public:
-		DocumentationView(QWidget* parent);
-		
-		void showDocumentation(KSharedPtr< KDevelop::IDocumentation > doc);
+    Q_OBJECT
+    public:
+        DocumentationView(QWidget* parent);
+        
+        void showDocumentation(KSharedPtr< KDevelop::IDocumentation > doc);
+        
     public slots:
         void browseForward();
         void browseBack();
+        void changedSelection();
+        void changedProvider(int);
+        void changeProvider(const QModelIndex &);
+        
     private:
-        void replaceView(QWidget*);
+        void updateView();
         
         KToolBar* mActions;
         QAction* mForward;
         QAction* mBack;
+        KLineEdit* mIdentifiers;
         QList< KSharedPtr< KDevelop::IDocumentation > > mHistory;
         QList< KSharedPtr< KDevelop::IDocumentation > >::iterator mCurrent;
+        QComboBox* mProviders;
+        ProvidersModel* mProvidersModel;
 };
 
 #endif // DOCUMENTATIONVIEW_H
