@@ -51,11 +51,19 @@ public:
 
     /*! @p status must implement KDevelop::IStatus */
     void registerStatus(QObject* status);
-
+    
     virtual QWidget* customButtonForAreaSwitcher( Sublime::Area* area );
+
+public Q_SLOTS:
+    /*! Shows an error message in the status bar.
+        @p message The message
+        @p timeout The timeout in milliseconds how long to show the message */
+    void showErrorMessage(const QString& message, int timeout);
+        
     
 public Q_SLOTS:
     virtual void setVisible( bool visible );
+    void configureShortcuts();
 
 Q_SIGNALS:
     void finishedLoading();
@@ -74,6 +82,10 @@ protected:
     virtual void cleanup();
     virtual void initializeStatusBar();
 
+protected Q_SLOTS:
+    virtual void tabContextMenuRequested(Sublime::View* , KMenu* );
+    virtual void dockBarContextMenuRequested(Qt::DockWidgetArea, const QPoint&);
+
 private Q_SLOTS:
     void documentActivated( KDevelop::IDocument* document );
     void documentStateChanged( KDevelop::IDocument* document );
