@@ -31,6 +31,8 @@ Boston, MA 02110-1301, USA.
 #include "mainwindow_p.h"
 #include "sessiondialog.h"
 
+#include "mainwindow.h"
+
 namespace KDevelop {
 
 // merge the gotoNext and gotoPrev code, to prevent copy/paste errors
@@ -81,14 +83,6 @@ void MainWindowPrivate::newToolbarConfig()
     m_mainWindow->applyMainWindowSettings( KConfigGroup(KGlobal::config(), "MainWindow") );
 }
 
-void MainWindowPrivate::toggleStatusbar()
-{
-    KToggleAction * action =
-        qobject_cast< KToggleAction*>( actionCollection() ->action(
-                                "options_show_statusbar" ) );
-    m_mainWindow->statusBar()->setHidden( !action->isChecked() );
-}
-
 void MainWindowPrivate::stopButtonPressed()
 {}
 
@@ -181,5 +175,30 @@ void MainWindowPrivate::showAboutPlatform()
     dlg.exec();
 }
 
+void MainWindowPrivate::contextMenuFileNew()
+{
+    m_mainWindow->activateView(m_tabView);
+    fileNew();
+}
+
+void MainWindowPrivate::contextMenuSplitHorizontal()
+{
+    m_mainWindow->activateView(m_tabView);
+    splitHorizontal();
+}
+
+void MainWindowPrivate::contextMenuSplitVertical()
+{
+    m_mainWindow->activateView(m_tabView);
+    splitVertical();
+}
+
+void MainWindowPrivate::reloadAll()
+{
+    foreach ( IDocument* doc, Core::self()->documentController()->openDocuments() ) {
+        doc->reload();
+    }
+}
+
 }
 
