@@ -43,10 +43,12 @@ Boston, MA 02110-1301, USA.
 #include <kactioncollection.h>
 #include <ktoolbarpopupaction.h>
 #include <knotifyconfigwidget.h>
+#include <kxmlguiclient.h>
 
 #include <sublime/area.h>
 #include <sublime/view.h>
 #include <sublime/document.h>
+#include <sublime/tooldocument.h>
 
 #include <interfaces/iplugin.h>
 
@@ -59,6 +61,8 @@ Boston, MA 02110-1301, USA.
 #include "mainwindow.h"
 #include "workingsetcontroller.h"
 
+#include "textdocument.h"
+
 namespace KDevelop {
 
 bool MainWindowPrivate::s_quitRequested = false;
@@ -86,14 +90,48 @@ void MainWindowPrivate::addPlugin( IPlugin *plugin )
     kDebug() << "add plugin" << plugin << plugin->componentData().componentName();
     Q_ASSERT( plugin );
 
-    m_mainWindow->guiFactory()->addClient( plugin );
+    //The direct plugin client can only be added to the first mainwindow
+    if(m_mainWindow == Core::self()->uiControllerInternal()->mainWindows()[0])
+        m_mainWindow->guiFactory()->addClient( plugin );
+    
+    Q_ASSERT(!m_pluginCustomClients.contains(plugin));
+    
+    KXMLGUIClient* ownClient = plugin->createGUIForMainWindow(m_mainWindow);
+    if(ownClient) {
+        m_pluginCustomClients[plugin] = ownClient;
+        connect(plugin, SIGNAL(destroyed(QObject*)), SLOT(pluginDestroyed(QObject*)));
+        m_mainWindow->guiFactory()->addClient(ownClient);
+    }
+    
     setupAreaSelectorActions();
 }
 
+void MainWindowPrivate::pluginDestroyed(QObject* pluginObj)
+{
+    IPlugin* plugin = static_cast<IPlugin*>(pluginObj);
+    Q_ASSERT(m_pluginCustomClients.contains(plugin));
+    m_mainWindow->guiFactory()->removeClient( m_pluginCustomClients[plugin] );
+    delete m_pluginCustomClients[plugin];
+    m_pluginCustomClients.remove(plugin);
+}
+
+MainWindowPrivate::~MainWindowPrivate()
+{
+    foreach(KXMLGUIClient* client, m_pluginCustomClients.values())
+        delete client;
+}
+
 void MainWindowPrivate::removePlugin( IPlugin *plugin )
 {
     Q_ASSERT( plugin );
 
+    if(m_pluginCustomClients.contains(plugin)) {
+        m_mainWindow->guiFactory()->removeClient( m_pluginCustomClients[plugin] );
+        delete m_pluginCustomClients[plugin];
+        m_pluginCustomClients.remove(plugin);
+        disconnect(plugin, SIGNAL(destroyed(QObject*)), this, SLOT(pluginDestroyed(QObject*)));
+    }
+    
     m_mainWindow->guiFactory()->removeClient( plugin );
     setupAreaSelectorActions();
 }
@@ -184,12 +222,6 @@ void MainWindowPrivate::setupActions()
     action->setWhatsThis( QString( "<b>%1</b><p>%2</p>" ).arg( text ).arg(
                               i18n( "Lets you customize %1.", app ) ) );
 
-    action = KStandardAction::showStatusbar( this, SLOT( toggleStatusbar() ),
-                                        actionCollection());
-    action->setText( i18n( "Show &Status Bar" ) );
-    action->setToolTip( i18n( "Show status bar" ) );
-    action->setWhatsThis( i18n( "<b>Show status bar</b><p>Hides or shows the status bar.</p>" ) );
-
     action = actionCollection()->addAction( "configure_sessions", this, SLOT( configureSessions() ) );
     action->setText( i18n("Configure Sessions...") );
     action->setToolTip( i18n("Create/Delete/Activate Sessions") );
@@ -279,7 +311,8 @@ void MainWindowPrivate::setupActions()
     action = actionCollection()->addAction( "file_new" );
     action->setIcon(KIcon("document-new"));
     action->setShortcut( Qt::CTRL + Qt::Key_N );
-    action->setText( i18n( "&New File" ) );
+    action->setText( i18n( "&New" ) );
+    action->setIconText( i18nc( "Shorter Text for 'New File' shown in the toolbar", "New") );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( fileNew() ) );
     action->setToolTip( i18n( "New File" ) );
     action->setWhatsThis( i18n( "<b>New File</b><p>Creates an empty file.</p>" ) );
@@ -362,6 +395,65 @@ void MainWindowPrivate::registerStatus(QObject* status)
     m_statusBar->registerStatus(status);
 }
 
+
+void MainWindowPrivate::showErrorMessage(QString message, int timeout)
+{
+    m_statusBar->showErrorMessage(message, timeout);
+}
+
+void MainWindowPrivate::tabContextMenuRequested(Sublime::View* view, KMenu* menu)
+{
+    m_tabView = view;
+
+    QAction* action;
+
+    action = menu->addAction(KIcon("view-split-top-bottom"), i18n("Split View Top/Bottom"));
+    connect(action, SIGNAL(triggered(bool)), this, SLOT(contextMenuSplitHorizontal()));
+
+    action = menu->addAction(KIcon("view-split-left-right"), i18n("Split View Left/Right"));
+    connect(action, SIGNAL(triggered(bool)), this, SLOT(contextMenuSplitVertical()));
+    menu->addSeparator();
+
+    action = menu->addAction(KIcon("document-new"), i18n("New File"));
+
+    connect(action, SIGNAL(triggered(bool)), this, SLOT(contextMenuFileNew()));
+
+    if ( TextDocument* doc = dynamic_cast<TextDocument*>(view->document()) ) {
+        action = menu->addAction(KIcon("view-refresh"), i18n("Reload"));
+        connect(action, SIGNAL(triggered(bool)), doc, SLOT(reload()));
+
+        action = menu->addAction(KIcon("view-refresh"), i18n("Reload All"));
+        connect(action, SIGNAL(triggered(bool)), this, SLOT(reloadAll()));
+    }
+}
+
+void MainWindowPrivate::dockBarContextMenuRequested(Qt::DockWidgetArea area, const QPoint& position)
+{
+    KMenu menu;
+    menu.addTitle(KIcon("window-new"), i18n("Add Tool View"));
+    QMap<IToolViewFactory*, Sublime::ToolDocument*> factories =
+        Core::self()->uiControllerInternal()->factoryDocuments();
+    QMap<QAction*, IToolViewFactory*> actionToFactory;
+    if ( !factories.isEmpty() ) {
+        for (QMap<IToolViewFactory*, Sublime::ToolDocument*>::const_iterator it = factories.constBegin();
+                it != factories.constEnd(); ++it)
+        {
+            QAction* action = menu.addAction(it.value()->statusIcon(), it.value()->title());
+            action->setIcon(it.value()->statusIcon());
+            actionToFactory.insert(action, it.key());
+        }
+    }
+
+    QAction* triggered = menu.exec(position);
+    if ( !triggered ) {
+        return;
+    }
+    Core::self()->uiControllerInternal()->addToolViewToDockArea(
+        triggered->text(), actionToFactory[triggered],
+        area
+    );
+}
+
 }
 
 #include "mainwindow_actions.cpp"
