@@ -28,6 +28,7 @@ Boston, MA 02110-1301, USA.
 #include <kxmlguiclient.h>
 
 class KActionCollection;
+class KMenu;
 
 namespace Sublime {
 class View;
@@ -62,10 +63,13 @@ public:
 
     void setupAreaSelectorActions();
     void setupAreaSelector();
+    
+    void tabContextMenuRequested(Sublime::View *view, KMenu* menu);
+    void dockBarContextMenuRequested(Qt::DockWidgetArea area, const QPoint& position);
 
 public Q_SLOTS:
     void addPlugin( KDevelop::IPlugin *plugin );
-    void removePlugin( IPlugin *plugin );
+    void removePlugin( KDevelop::IPlugin *plugin );
 
     void activePartChanged(KParts::Part *part);
     void changeActiveView(Sublime::View *view);
@@ -94,8 +98,6 @@ public Q_SLOTS:
     void stopButtonPressed();
     void showMenuBar();
 
-    void toggleStatusbar();
-
     void quitAll();
 
 //    void fixToolbar();
@@ -110,7 +112,16 @@ public Q_SLOTS:
     void showErrorMessage(QString message, int timeout);
     void pluginDestroyed(QObject*);
 
-    
+    /// the following slots always activate the m_tabView before calling the normal slot above
+    /// @see m_tabView
+    /// @see tabContextMenuRequested
+    void contextMenuFileNew();
+    void contextMenuSplitHorizontal();
+    void contextMenuSplitVertical();
+
+    /// reload all open documents
+    void reloadAll();
+
 private:
     KActionCollection *actionCollection();
 
@@ -122,6 +133,8 @@ private:
     QMap<IPlugin*, KXMLGUIClient*> m_pluginCustomClients;
     
     static bool s_quitRequested;
+    /// the view of the tab that got it's context menu connected
+    Sublime::View* m_tabView;
 };
 
 }
