@@ -60,49 +60,10 @@ public:
 
     KTextEditor::Editor *m_textEditor;
 
-    KParts::Factory *findPartFactory( const QString &mimeType,
-                                      const QString &partType,
-                                      const QString &preferredName = QString() )
-
-    {
-        KService::List offers = KMimeTypeTrader::self() ->query(
-                                    mimeType,
-                                    "KParts/ReadOnlyPart",
-                                    QString( "'%1' in ServiceTypes" ).arg( partType ) );
-
-        if ( offers.count() > 0 )
-        {
-            KService::Ptr ptr;
-            // if there is a preferred plugin we'll take it
-            if ( !preferredName.isEmpty() )
-            {
-                KService::List::ConstIterator it;
-                for ( it = offers.constBegin(); it != offers.constEnd(); ++it )
-                {
-                    if ( ( *it ) ->desktopEntryName() == preferredName )
-                    {
-                        ptr = ( *it );
-                    }
-                }
-            }
-            // else we just take the first in the list
-            if ( !ptr )
-            {
-                ptr = offers.first();
-            }
-            KParts::Factory *factory = static_cast<KParts::Factory*>(
-                                           KLibLoader::self() ->factory(
-                                               QFile::encodeName( ptr->library() ) ) );
-            return factory;
-        }
-
-        return 0;
-    }
-
 };
 
 PartController::PartController(Core *core, QWidget *toplevel)
-        : KParts::PartManager( toplevel, 0 ), d(new PartControllerPrivate)
+        : IPartController( toplevel ), d(new PartControllerPrivate)
 
 {
     setObjectName("PartController");
@@ -147,7 +108,7 @@ KTextEditor::Document* PartController::createTextPart(const QString &encoding)
 {
     if (!d->m_textEditor)
     {
-        KTextEditor::Factory * editorFactory = qobject_cast<KTextEditor::Factory*>(d->findPartFactory(
+        KTextEditor::Factory * editorFactory = qobject_cast<KTextEditor::Factory*>(findPartFactory(
             "text/plain",
             "KTextEditor/Document",
             "KTextEditor::Editor" ));
@@ -159,7 +120,7 @@ KTextEditor::Document* PartController::createTextPart(const QString &encoding)
     if ( !encoding.isNull() )
     {
         KParts::OpenUrlArguments args = doc->arguments();
-        args.setMimeType( QString( "text/plain;" ) + encoding );
+        args.setMimeType( QString::fromLatin1( "text/plain;" ) + encoding );
         doc->setArguments( args );
     }
 
@@ -191,7 +152,7 @@ KParts::Part* PartController::createPart( const QString & mimeType,
         const QString & className,
         const QString & preferredName )
 {
-    KParts::Factory * editorFactory = d->findPartFactory(
+    KParts::Factory * editorFactory = findPartFactory(
                                           mimeType,
                                           partType,
                                           preferredName );
@@ -207,49 +168,40 @@ KParts::Part* PartController::createPart( const QString & mimeType,
     return 0;
 }
 
-KParts::Part* PartController::createPart( const KUrl & url )
+bool PartController::canCreatePart(const KUrl& url)
 {
-    KMimeType::Ptr mimeType;
+    if (!url.isValid()) return false;
+
+    QString mimeType;
+    if ( url.isEmpty() )
+        mimeType = QString::fromLatin1("text/plain");
+    else
+        mimeType = KMimeType::findByUrl( url )->name();
+
+    KService::List offers = KMimeTypeTrader::self()->query(
+                                mimeType,
+                                "KParts/ReadOnlyPart" );
+
+    return offers.count() > 0;
+}
+
+KParts::Part* PartController::createPart( const KUrl & url, const QString& preferredPart )
+{
+    kDebug() << "creating part with url" << url << "and pref part:" << preferredPart;
+    QString mimeType;
     if ( url.isEmpty() )
         //create a part for empty text file
-        mimeType = KMimeType::mimeType("text/plain");
+        mimeType = QString::fromLatin1("text/plain");
     else if ( !url.isValid() )
         return 0;
     else
-        mimeType = KMimeType::findByUrl( url );
+        mimeType = KMimeType::findByUrl( url )->name();
 
-    QString className;
-    QString services[] =
-        {
-            "KParts/ReadWritePart", "KParts/ReadOnlyPart"
-        };
-
-    QString classNames[] =
-        {
-            "KParts::ReadWritePart", "KParts::ReadOnlyPart"
-        };
-    KParts::Factory *editorFactory = 0;
-    for ( uint i = 0; i < 2; ++i )
+    KParts::Part* part = createPart( mimeType, preferredPart );
+    if( part )
     {
-        editorFactory = d->findPartFactory( mimeType->name(), services[ i ] );
-        if ( editorFactory )
-        {
-            className = classNames[ i ];
-            break;
-        }
-    }
-
-    if ( !className.isEmpty() && editorFactory )
-    {
-        KParts::Part * part = editorFactory->createPart(
-                                  0,
-                                  this,
-                                  className.toLatin1() );
-        if( part )
-        {
-            readOnly( part ) ->openUrl( url );
-            return part;
-        }
+        readOnly( part ) ->openUrl( url );
+        return part;
     }
 
     return 0;
