@@ -48,7 +48,6 @@
 #include <interfaces/iplugin.h>
 #include <interfaces/iplugincontroller.h>
 #include <interfaces/iruncontroller.h>
-#include <project/importprojectjob.h>
 #include <project/projectmodel.h>
 #include <language/duchain/indexedstring.h>
 #include <vcs/interfaces/ibasicversioncontrol.h>
@@ -61,6 +60,77 @@
 namespace KDevelop
 {
 
+class ProjectProgress : public QObject, public IStatus
+{
+    Q_OBJECT
+    Q_INTERFACES(KDevelop::IStatus)
+    
+    public:
+        ProjectProgress();
+        virtual ~ProjectProgress();
+        virtual QString statusName() const;
+        
+        /*! Show indeterminate mode progress bar */
+        void setBuzzy();
+        
+        /*! Hide progress bar */
+        void setDone();
+        
+        QString projectName;
+        
+    private Q_SLOTS:
+        void slotClean();
+        
+    Q_SIGNALS:
+        void clearMessage(KDevelop::IStatus*);
+        void showMessage(KDevelop::IStatus*,const QString & message, int timeout = 0);
+        void showErrorMessage(const QString & message, int timeout = 0);
+        void hideProgress(KDevelop::IStatus*);
+        void showProgress(KDevelop::IStatus*,int minimum, int maximum, int value);
+};
+    
+    
+    
+ProjectProgress::ProjectProgress()
+{
+}
+
+ProjectProgress::~ProjectProgress()
+{
+}
+
+QString ProjectProgress::statusName() const
+{
+    return i18n("Loading Project ") + projectName;
+}
+
+void ProjectProgress::setBuzzy()
+{
+    kDebug() << "showing busy prorgess" << statusName();
+    // show an indeterminate progressbar
+    emit showProgress(this, 0,0,0);
+    emit showMessage(this, i18n("Loading") + " " + projectName);
+}
+
+
+void ProjectProgress::setDone()
+{
+    kDebug() << "showing done progress" << statusName();
+    // first show 100% bar for a second, then hide.
+    emit showProgress(this, 0,1,1);
+    QTimer* t = new QTimer;
+    t->setSingleShot( true );
+    t->setInterval( 1000 );
+    connect(t, SIGNAL(timeout()),SLOT(slotClean()));
+    t->start();
+}
+
+void ProjectProgress::slotClean()
+{
+    emit hideProgress(this);
+    emit clearMessage(this);
+}
+
 class ProjectPrivate
 {
 public:
@@ -79,9 +149,11 @@ public:
     QSet<KDevelop::IndexedString> fileSet;
     bool reloading;
     bool scheduleReload;
+    ProjectProgress* progress;
 
     void reloadDone()
     {
+        progress->setDone();
         reloading = false;
         Core::self()->projectController()->projectModel()->appendRow(topItem);
         if (scheduleReload) {
@@ -175,6 +247,7 @@ public:
 
     void importDone( KJob* )
     {
+        progress->setDone();
         ProjectController* projCtrl = Core::self()->projectControllerInternal();
         projCtrl->projectModel()->appendRow(topItem);
         projCtrl->projectImportingFinished( project );
@@ -264,6 +337,7 @@ public:
     {
         // helper method for open()
         name = projectGroup.readEntry( "Name", projectFileUrl.fileName() );
+        progress->projectName = name;
         if( Core::self()->projectController()->isProjectNameUsed( name ) ) 
         {
             kWarning() << "Trying to open a project with a name thats already used by another open project";
@@ -367,10 +441,13 @@ Project::Project( QObject *parent )
     d->vcsPlugin = 0;
     d->reloading = false;
     d->scheduleReload = false;
+    d->progress = new ProjectProgress;
+    Core::self()->uiController()->registerStatus( d->progress );
 }
 
 Project::~Project()
 {
+    delete d->progress;
     delete d;
 }
 
@@ -420,7 +497,8 @@ void Project::reloadModel()
             return;
     }
 
-    ImportProjectJob* importJob = new ImportProjectJob( d->topItem, iface );
+    d->progress->setBuzzy();
+    KJob* importJob = iface->createImportJob(d->topItem );
     connect(importJob, SIGNAL(finished(KJob*)), SLOT(reloadDone()));
     Core::self()->runController()->registerJob( importJob );
 }
@@ -448,7 +526,8 @@ bool Project::open( const KUrl& projectFileUrl_ )
     }
 
     d->loadVersionControlPlugin(projectGroup);
-    ImportProjectJob* importJob = new ImportProjectJob( d->topItem, iface );
+    d->progress->setBuzzy();
+    KJob* importJob = iface->createImportJob(d->topItem );
     connect( importJob, SIGNAL( result( KJob* ) ), this, SLOT( importDone( KJob* ) ) );
     Core::self()->runController()->registerJob( importJob );
     return true;
@@ -516,7 +595,7 @@ QList<ProjectFolderItem*> Project::foldersForUrl(const KUrl& url) const
     QList<ProjectFolderItem*> items;
     foreach(ProjectBaseItem* item,  d->itemsForUrl( url ) )
     {
-        if( item->type() == ProjectBaseItem::Folder )
+        if( item->type() == ProjectBaseItem::Folder || item->type() == ProjectBaseItem::BuildFolder )
             items << dynamic_cast<ProjectFolderItem*>( item );
     }
     return items;
@@ -532,7 +611,6 @@ int Project::fileCount() const
 
 KUrl Project::relativeUrl( const KUrl& absolute ) const
 {
-    kDebug() << "Creating relative url between: " << folder() << absolute;
     return KUrl::relativeUrl( folder(), absolute );
 }
 
@@ -595,4 +673,4 @@ QSet<IndexedString> Project::fileSet() const
 } // namespace KDevelop
 
 #include "project.moc"
-
+#include "moc_project.cpp"
