@@ -47,7 +47,6 @@ Boston, MA 02110-1301, USA.
 #include <ksettings/dialog.h>
 #include <kio/netaccess.h>
 #include <kstandarddirs.h>
-#include <knotification.h>
 
 #include <ksettings/dispatcher.h>
 
@@ -345,8 +344,8 @@ void ProjectController::setupActions()
     action = ac->addAction( "project_open" );
     action->setText(i18n( "Open / Import Project..." ) );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( openProject() ) );
-    action->setToolTip( i18n( "Open / Import Oroject" ) );
-    action->setWhatsThis( i18n( "<b>Open / Import project</b><p>Open an existing KDevelop 4 project or import an existing Project into KDevelop 4. This entry allows to select a KDevelop4 project file or an existing directory to open it in KDevelop. When opening an existing directory that doesn't yet have a KDevelop4 project file the file will be created.</p>" ) );
+    action->setToolTip( i18n( "Open / Import Project" ) );
+    action->setWhatsThis( i18n( "<b>Open / Import project</b><p>Open an existing KDevelop 4 project or import an existing Project into KDevelop 4. This entry allows to select a KDevelop4 project file or an existing directory to open it in KDevelop. When opening an existing directory that does not yet have a KDevelop4 project file, the file will be created.</p>" ) );
     action->setIcon(KIcon("project-open"));
 
 //    action = ac->addAction( "project_close" );
@@ -362,17 +361,19 @@ void ProjectController::setupActions()
     action->setToolTip( i18n( "Close all currently open projects" ) );
     action->setWhatsThis( i18n( "<b>Close all projects</b><p>Closes all of the currently open projects.</p>" ) );
     action->setEnabled( false );
-    action->setIcon(KIcon("project-close"));
+    action->setIcon(KIcon("project-development-close-all"));
 
     d->m_closeProject = action = ac->addAction( "project_close" );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( closeSelectedProjects() ) );
     action->setText( i18n( "Close Project(s)" ) );
+    action->setIcon( KIcon( "project-development-close" ) );
     action->setToolTip( i18n( "Closes all currently selected projects" ) );
     action->setEnabled( false );
 
     d->m_openConfig = action = ac->addAction( "project_open_config" );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( openProjectConfig() ) );
     action->setText( i18n( "Open Configuration..." ) );
+    action->setIcon( KIcon("configure") );
     action->setEnabled( false );
 
     KSharedConfig * config = KGlobal::config().data();
@@ -390,7 +391,7 @@ void ProjectController::setupActions()
     
     KAction* openProjectForFileAction = new KAction( this );
     ac->addAction("project_open_for_file", openProjectForFileAction);
-    openProjectForFileAction->setText("Open Project for Current File");
+    openProjectForFileAction->setText(i18n("Open Project for Current File"));
     connect( openProjectForFileAction, SIGNAL(triggered(bool)), SLOT(openProjectForUrlSlot(bool)));
 }
 
@@ -564,25 +565,10 @@ void ProjectController::openProject( const KUrl &projectFile )
         }
     }
 
-    KNotification* ev=new KNotification("LoadingProject", Core::self()->uiControllerInternal()->activeMainWindow());
-    ev->setText(i18n( "Loading Project: %1", url.prettyUrl() ));
-    ev->setComponentData(KGlobal::mainComponent());
-    ev->sendEvent();
-
-    //FIXME Create the hidden directory if it doesn't exist
-    if ( loadProjectPart() )
-    {
-        //The project file has been opened.
-        //Now we can load settings for all of the Core objects including this one!!
-//         Core::loadSettings();
-        d->m_core->pluginControllerInternal()->loadProjectPlugins();
-    } else {
-        KMessageBox::error(Core::self()->uiControllerInternal()->activeMainWindow(),
-                           i18n("Cannot Load Projects View plugin, aborting."));
-        return;
-    }
+    d->m_core->pluginControllerInternal()->loadProjectPlugins();
 
     Project* project = new Project();
+    emit projectAboutToBeOpened( project );
     if ( !project->open( url ) )
     {
         KMessageBox::error(Core::self()->uiControllerInternal()->activeMainWindow(),
@@ -627,28 +613,23 @@ void ProjectController::projectImportingFinished( IProject* project )
     d->m_currentlyOpening.removeAll(project->projectFileUrl());
     emit projectOpened( project );
 
-    KUrl::List parseList;
     if (parseAllProjectSources())
     {
         KJob* parseProjectJob = new KDevelop::ParseProjectJob(project);
         ICore::self()->runController()->registerJob(parseProjectJob);
-    } else
-    {
-        // Add all currently open files that belong to the project to the background-parser,
-        // since more information may be available for parsing them now(Like include-paths).
-        foreach(IDocument* document, Core::self()->documentController()->openDocuments())
-        {
-            if(!project->filesForUrl(document->url()).isEmpty())
-            {
-                parseList.append(document->url());
-            }
-        }
-        Core::self()->languageController()->backgroundParser()->addDocumentList( parseList, KDevelop::TopDUContext::AllDeclarationsContextsAndUses, 10 );
     }
-    KNotification* ev=new KNotification("ProjectLoadedSuccessfully", Core::self()->uiControllerInternal()->activeMainWindow());
-    ev->setText(i18n( "Project loaded: %1", project->name() ));
-    ev->setComponentData(KGlobal::mainComponent());
-    ev->sendEvent();
+    
+    KUrl::List parseList;
+    // Add all currently open files that belong to the project to the background-parser,
+    // since more information may be available for parsing them now(Like include-paths).
+    foreach(IDocument* document, Core::self()->documentController()->openDocuments())
+    {
+        if(!project->filesForUrl(document->url()).isEmpty())
+        {
+            parseList.append(document->url());
+        }
+    }
+    Core::self()->languageController()->backgroundParser()->addDocumentList( parseList, KDevelop::TopDUContext::AllDeclarationsContextsAndUses, 10 );
 }
 
 // helper method for closeProject()
@@ -725,12 +706,6 @@ void ProjectController::closeProject(IProject* proj)
     return;
 }
 
-bool ProjectController::loadProjectPart()
-{
-
-    return true;
-}
-
 ProjectModel* ProjectController::projectModel()
 {
     return d->model;
@@ -803,15 +778,23 @@ KUrl ProjectController::projectsBaseDirectory() const
                                      KUrl( QDir::homePath()+"/projects" ) );
 }
 
-QString ProjectController::prettyFileName(KUrl url) const
+QString ProjectController::prettyFileName(KUrl url, FormattingOptions format) const
 {
     IProject* project = Core::self()->projectController()->findProjectForUrl(url);
     QString prefixText = url.upUrl().pathOrUrl(KUrl::AddTrailingSlash);
-    if(project)
-        prefixText = "<i>" +  project->name() + "/</i>" + project->relativeUrl(url.upUrl()).path(KUrl::AddTrailingSlash);
-
-    return prefixText + "<b>" + url.fileName() + "</b>";
-    
+    if (project) {
+        if (format == FormatHtml) {
+            prefixText = "<i>" +  project->name() + "/</i>";
+        } else {
+            prefixText = project->name() + '/';
+        }
+        prefixText += project->relativeUrl(url.upUrl()).path(KUrl::AddTrailingSlash);
+    }
+    if (format == FormatHtml) {
+        return prefixText + "<b>" + url.fileName() + "</b>";
+    } else {
+        return prefixText + url.fileName();
+    }
 }
 
 }
