@@ -58,7 +58,7 @@ public:
     UiControllerPrivate(UiController *controller)
     : cfgDlg(0), areasRestored(false), m_controller(controller)
     {
-        Core::self()->workingSetControllerInternal()->initializeController(controller);
+        Core::self()->workingSetControllerInternal()->initializeController(m_controller);
         
         QMap<QString, Sublime::Position> desired;
 
@@ -71,11 +71,12 @@ public:
         desired["org.kdevelop.ContextBrowser"] = Sublime::Bottom;
         desired["org.kdevelop.debugger.ConsoleView"] = Sublime::Bottom;
         desired["org.kdevelop.KonsoleView"] = Sublime::Bottom;
+        desired["org.kdevelop.SnippetView"] = Sublime::Right;
         Sublime::Area* a =
             new Sublime::Area(m_controller, "code", i18n("Code"));
         a->setDesiredToolViews(desired);
         a->setIconName("document-edit");
-        controller->addDefaultArea(a);
+        m_controller->addDefaultArea(a);
 
         desired.clear();
         desired["org.kdevelop.debugger.VariablesView"] = Sublime::Left;
@@ -86,20 +87,21 @@ public:
         a = new Sublime::Area(m_controller, "debug", i18n("Debug"));
         a->setDesiredToolViews(desired);
         a->setIconName("tools-report-bug");
-        controller->addDefaultArea(a);
+        m_controller->addDefaultArea(a);
 
         desired.clear();
-        desired["org.kdevelop.QTestTestRunner"] = Sublime::Right;
-        desired["org.kdevelop.CoverageReport"] = Sublime::Right;
-        a = new Sublime::Area(m_controller, "test", i18n("Test"));
+        desired["org.kdevelop.ProjectsView"] = Sublime::Left;
+        desired["org.kdevelop.PatchReview"] = Sublime::Bottom;
+        
+        a = new Sublime::Area(m_controller, "review", i18n("Review"));
         a->setDesiredToolViews(desired);
         a->setIconName("applications-engineering");
-        controller->addDefaultArea(a);
+        m_controller->addDefaultArea(a);
 
         if(!(Core::self()->setupFlags() & Core::NoUi)) 
         {
-            defaultMainWindow = new MainWindow(controller);
-            controller->addMainWindow(defaultMainWindow);
+            defaultMainWindow = new MainWindow(m_controller);
+            m_controller->addMainWindow(defaultMainWindow);
             activeSublimeWindow = defaultMainWindow;
         }
         else
@@ -496,6 +498,10 @@ void UiController::loadAllAreas(KSharedConfig::Ptr config)
         KConfigGroup mainWindowConfig(&uiConfig,
                                       QString("Main Window %1").arg(w));
         QString currentArea = mainWindowConfig.readEntry("currentArea", "");
+        
+        if(currentArea == "test") ///@todo The area was renamed, and it will lead to a crash, so this is a dirty temporary workaround
+            currentArea = "review";
+        
         Sublime::MainWindow *mw = mainWindows()[w];
 
         /* We loop over default areas.  This means that if
@@ -545,6 +551,16 @@ void UiController::loadAllAreas(KSharedConfig::Ptr config)
     d->areasRestored = true;
 }
 
+void UiController::addToolViewToDockArea(const QString& name,
+                                         IToolViewFactory* factory,
+                                         Qt::DockWidgetArea area)
+{
+    ///TODO: we should probably add a bool forcePosition member to
+    /// Area::addToolView(), to force adding at the given position.
+    Sublime::View* view = addToolViewToArea(factory, d->factoryDocuments[factory], activeArea());
+    activeArea()->moveToolView(view, Sublime::dockAreaToPosition(area));
+}
+
 void UiController::addToolViewIfWanted(IToolViewFactory* factory,
                            Sublime::ToolDocument* doc,
                            Sublime::Area* area)
@@ -555,7 +571,7 @@ void UiController::addToolViewIfWanted(IToolViewFactory* factory,
     }
 }
 
-void UiController::addToolViewToArea(IToolViewFactory* factory,
+Sublime::View* UiController::addToolViewToArea(IToolViewFactory* factory,
                                      Sublime::ToolDocument* doc,
                                      Sublime::Area* area)
 {
@@ -568,6 +584,7 @@ void UiController::addToolViewToArea(IToolViewFactory* factory,
             SLOT(raiseToolView(Sublime::View*)));
 
     factory->viewCreated(view);
+    return view;
 }
 
 void UiController::registerStatus(QObject* status)
@@ -625,6 +642,11 @@ void UiController::popUpAssistant(const KDevelop::IAssistant::Ptr& assistant)
     }
 }
 
+const QMap< IToolViewFactory*, Sublime::ToolDocument* >& UiController::factoryDocuments() const
+{
+    return d->factoryDocuments;
+}
+
 void UiController::assistantAction1() {
     if(d->currentShownAssistant)
         d->currentShownAssistant->executeAction1();
