@@ -35,6 +35,8 @@
 #include <kglobal.h>
 #include <kdeversion.h>
 #include <kacceleratormanager.h>
+#include <kmenu.h>
+#include <kicon.h>
 
 #include "view.h"
 #include "document.h"
@@ -135,6 +137,7 @@ Container::Container(QWidget *parent)
     m_tabBarLayout->setSpacing(0);
 
     d->tabBar = new KTabBar(this);
+    d->tabBar->setContextMenuPolicy(Qt::CustomContextMenu);
     m_tabBarLayout->addWidget(d->tabBar);
     d->fileStatus = new QLabel( this );
     d->fileStatus->setFixedSize( QSize( 16, 16 ) );
@@ -152,18 +155,13 @@ Container::Container(QWidget *parent)
     connect(d->tabBar, SIGNAL(closeRequest(int)), this, SLOT(closeRequest(int)));
     connect(d->tabBar, SIGNAL(tabMoved(int,int)), this, SLOT(tabMoved(int, int)));
     connect(d->tabBar, SIGNAL(wheelDelta(int)), this, SLOT(wheelScroll(int)));
+    connect(d->tabBar, SIGNAL(contextMenu(int,QPoint)), this, SLOT(contextMenu(int,QPoint)));
 
     KConfigGroup group = KGlobal::config()->group("UiSettings");
     setTabBarHidden(group.readEntry("TabBarVisibility", 1) == 0);
-#if KDE_VERSION < KDE_MAKE_VERSION(4,2,60)
-    d->tabBar->setHoverCloseButton(true);
-    d->tabBar->setCloseButtonEnabled(true);
-    d->tabBar->setTabReorderingEnabled(true);
-#else
     d->tabBar->setTabsClosable(true);
     d->tabBar->setMovable(true);
     d->tabBar->setExpanding(false);
-#endif
 
     setOpenAfterCurrent(group.readEntry("TabBarOpenAfterCurrent", 1) == 1);
 }
@@ -370,6 +368,39 @@ void Container::tabMoved(int from, int to)
     d->viewForWidget[w]->notifyPositionChanged(to);
 }
 
+void Container::contextMenu( int currentTab, QPoint pos )
+{
+    KMenu menu;
+
+    emit tabContextMenuRequested(viewForWidget(widget(currentTab)), &menu);
+
+    menu.addSeparator();
+    QAction* closeTabAction = menu.addAction( KIcon("document-close"), i18n( "Close File" ) );
+    QAction* closeOtherTabsAction = menu.addAction( KIcon("document-close"), i18n( "Close Other Files" ) );
+
+    QAction* triggered = menu.exec(pos);
+
+    if (triggered) {
+        if ( triggered == closeTabAction ) {
+            closeRequest(currentTab);
+        } else if ( triggered == closeOtherTabsAction ) {
+            // activate the remaining tab
+            widgetActivated(currentTab);
+            // first get the widgets to be closed since otherwise the indices will be wrong
+            QList<QWidget*> otherTabs;
+            for ( int i = 0; i < count(); ++i ) {
+                if ( i != currentTab ) {
+                    otherTabs << widget(i);
+                }
+            }
+            // finally close other tabs
+            foreach( QWidget* tab, otherTabs ) {
+                closeRequest(tab);
+            }
+        } // else the action was handled by someone else
+    }
+}
+
 }
 
 #include "container.moc"
