@@ -50,6 +50,8 @@ IdealToolButton::IdealToolButton(Qt::DockWidgetArea area, QWidget *parent)
     setAutoRaise(true);
     setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     setIcons( "process-working-kde" );
+
+    setContextMenuPolicy(Qt::CustomContextMenu);
 }
 
 Qt::Orientation IdealToolButton::orientation() const
@@ -141,7 +143,7 @@ void IdealToolButton::paintEvent(QPaintEvent *event)
         opt.rect.setSize(QSize(opt.rect.height(), opt.rect.width()));
 
         QPixmap pix(opt.rect.width(), opt.rect.height());
-	QStylePainter painter(&pix, this);
+        QStylePainter painter(&pix, this);
         painter.fillRect(pix.rect(), opt.palette.background());
         painter.drawComplexControl(QStyle::CC_ToolButton, opt);
         painter.end();
@@ -163,11 +165,9 @@ void IdealToolButton::paintEvent(QPaintEvent *event)
 IdealButtonBarWidget::IdealButtonBarWidget(Qt::DockWidgetArea area, IdealMainWidget *parent)
     : QWidget(parent)
     , _area(area)
-    , _actions(new QActionGroup(this))
     , _corner(0)
 {
-    // TODO Only for now...
-    _actions->setExclusive(true);
+    setContextMenuPolicy(Qt::CustomContextMenu);
 
     if (area == Qt::BottomDockWidgetArea)
     {
@@ -212,7 +212,6 @@ KAction *IdealButtonBarWidget::addWidget(const QString& title, IdealDockWidget *
     connect(action, SIGNAL(toggled(bool)), this, SLOT(showWidget(bool)));
 
     addAction(action);
-    _actions->addAction(action);
 
     return action;
 }
@@ -239,6 +238,11 @@ Qt::Orientation IdealButtonBarWidget::orientation() const
     return Qt::Horizontal;
 }
 
+Qt::DockWidgetArea IdealButtonBarWidget::area() const
+{
+    return _area;
+}
+
 void IdealButtonBarWidget::showWidget(bool checked)
 {
     Q_ASSERT(parentWidget() != 0);
@@ -248,6 +252,19 @@ void IdealButtonBarWidget::showWidget(bool checked)
 
     IdealDockWidget *widget = _widgets.value(action);
     Q_ASSERT(widget);
+    
+    if ( checked ) {
+        // Make sure only one widget is visible at any time.
+        // The alternative to use a QActionCollection and setting that to "exclusive"
+        // has a big drawback: QActions in a collection that is exclusive cannot
+        // be un-checked by the user, e.g. in the View -> Tool Views menu.
+        foreach(QAction *otherAction, actions()) {
+            if ( otherAction != action && otherAction->isChecked() ) {
+                otherAction->setChecked(false);
+                break;
+            }
+        }
+    }
 
     parentWidget()->showDockWidget(widget, checked);
 }
@@ -283,6 +300,8 @@ void IdealButtonBarWidget::actionEvent(QActionEvent *event)
             connect(button, SIGNAL(toggled(bool)), action, SLOT(setChecked(bool)));
             connect(_widgets[action]->view(), SIGNAL(showProgressIndicator()), button, SLOT(showProgressIndicator()));
             connect(_widgets[action]->view(), SIGNAL(hideProgressIndicator()), button, SLOT(hideProgressIndicator()));
+            connect(button, SIGNAL(customContextMenuRequested(QPoint)),
+                    _widgets[action], SLOT(contextMenuRequested(QPoint)));
         }
     } break;
 
@@ -302,12 +321,12 @@ void IdealButtonBarWidget::actionEvent(QActionEvent *event)
     } break;
 
     case QEvent::ActionChanged: {
-	if (IdealToolButton *button = _buttons.value(action)) {
-	    button->setText(action->text());
-	    button->updateNormalIcon(action->icon());
-	    button->setShortcut(QKeySequence());
-	    Q_ASSERT(_widgets.contains(action));
-	    _widgets[action]->setWindowTitle(action->text());
+        if (IdealToolButton *button = _buttons.value(action)) {
+            button->setText(action->text());
+            button->updateNormalIcon(action->icon());
+            button->setShortcut(QKeySequence());
+            Q_ASSERT(_widgets.contains(action));
+            _widgets[action]->setWindowTitle(action->text());
         }
     } break;
 
@@ -336,32 +355,35 @@ IdealDockWidget::IdealDockWidget(QWidget *parent)
       m_docking_area(Qt::NoDockWidgetArea),
       m_maximized(false)
 {
+    setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(this, SIGNAL(customContextMenuRequested(QPoint)),
+            this, SLOT(contextMenuRequested(QPoint)));
 
     QAbstractButton *floatButton =
-	qFindChild<QAbstractButton *>(this, QLatin1String("qt_dockwidget_floatbutton"));
+    qFindChild<QAbstractButton *>(this, QLatin1String("qt_dockwidget_floatbutton"));
 
     QAbstractButton *closeButton =
-	qFindChild<QAbstractButton *>(this, QLatin1String("qt_dockwidget_closebutton"));
+    qFindChild<QAbstractButton *>(this, QLatin1String("qt_dockwidget_closebutton"));
 
     if (floatButton && closeButton) {
-	disconnect(floatButton, SIGNAL(clicked()), 0, 0);
-	disconnect(closeButton, SIGNAL(clicked()), 0, 0);
+    disconnect(floatButton, SIGNAL(clicked()), 0, 0);
+    disconnect(closeButton, SIGNAL(clicked()), 0, 0);
 
-	m_anchor = floatButton;
-	m_anchor->setCheckable(true);
-	m_anchor->setToolTip(i18n("Lock the tool"));
-	m_anchor->setWhatsThis(i18n("<b>Lock the tool</b><p>When a tool is unlocked, it "
-				    "will be automatically hidden when you click outside it. "
-				    "A locked tool will remain visible until you explicitly "
-				    "hide it, or switch to a different tool.</p>"));
-	connect(m_anchor, SIGNAL(toggled(bool)), SIGNAL(anchor(bool)));
+    m_anchor = floatButton;
+    m_anchor->setCheckable(true);
+    m_anchor->setToolTip(i18n("Lock the tool"));
+    m_anchor->setWhatsThis(i18n("<b>Lock the tool</b><p>When a tool is unlocked, it "
+                    "will be automatically hidden when you click outside it. "
+                    "A locked tool will remain visible until you explicitly "
+                    "hide it, or switch to a different tool.</p>"));
+    connect(m_anchor, SIGNAL(toggled(bool)), SIGNAL(anchor(bool)));
 
-	m_close = closeButton;
-	m_close->setToolTip(i18n("Remove the tool"));
-	m_close->setWhatsThis(i18n("<b>Remove the tool</b><p>Removes this tool completely. "
-				   "You can add the tool again by using the "
-				   "<tt>View->Add Tool View</tt> command.</p>"));
-	connect(m_close, SIGNAL(clicked(bool)), this, SLOT(slotRemove()));
+    m_close = closeButton;
+    m_close->setToolTip(i18n("Remove the tool"));
+    m_close->setWhatsThis(i18n("<b>Remove the tool</b><p>Removes this tool completely. "
+                   "You can add the tool again by using the "
+                   "<tt>View->Add Tool View</tt> command.</p>"));
+    connect(m_close, SIGNAL(clicked(bool)), this, SLOT(slotRemove()));
     }
 }
 
@@ -440,11 +462,15 @@ void IdealDockWidget::slotRemove()
     m_area->removeToolView(m_view);
 }
 
-void IdealDockWidget::contextMenuEvent(QContextMenuEvent *event)
+void IdealDockWidget::contextMenuRequested(const QPoint &point)
 {
+    QWidget* senderWidget = qobject_cast<QWidget*>(sender());
+    Q_ASSERT(senderWidget);
+
     KMenu menu;
 
-    menu.addTitle(i18n("Position"));
+    /// start position menu
+    QMenu* positionMenu = menu.addMenu(i18n("Position"));
 
     QActionGroup *g = new QActionGroup(this);
 
@@ -456,7 +482,7 @@ void IdealDockWidget::contextMenuEvent(QContextMenuEvent *event)
     QAction* actions[] = {left, bottom, right, top};
     for (int i = 0; i < 4; ++i)
     {
-        menu.addAction(actions[i]);
+        positionMenu->addAction(actions[i]);
         actions[i]->setCheckable(true);
     }
     if (m_docking_area == Qt::TopDockWidgetArea)
@@ -467,11 +493,29 @@ void IdealDockWidget::contextMenuEvent(QContextMenuEvent *event)
         left->setChecked(true);
     else
         right->setChecked(true);
+    /// end position menu
 
-    QAction* triggered = menu.exec(event->globalPos());
+    menu.addSeparator();
+    QAction* remove = menu.addAction(KIcon("dialog-close"), i18n("Remove"));
+    QAction* toggleAnchored;
+    if ( isAnchored() ) {
+        toggleAnchored = menu.addAction(KIcon("document-decrypt"), i18n("Unlock"));
+    } else {
+        toggleAnchored = menu.addAction(KIcon("document-encrypt"), i18n("Lock"));
+    }
+
+    QAction* triggered = menu.exec(senderWidget->mapToGlobal(point));
 
     if (triggered)
     {
+        if ( triggered == remove ) {
+            slotRemove();
+            return;
+        } else if ( triggered == toggleAnchored ) {
+            setAnchored(!isAnchored(), true);
+            return;
+        }
+
         Sublime::Position pos;
         if (triggered == left)
             pos = Sublime::Left;
@@ -498,15 +542,23 @@ IdealMainWidget::IdealMainWidget(MainWindow* parent, KActionCollection* ac)
     Q_UNUSED(ac);
     leftBarWidget = new IdealButtonBarWidget(Qt::LeftDockWidgetArea, this);
     leftBarWidget->hide();
+    connect(leftBarWidget, SIGNAL(customContextMenuRequested(QPoint)),
+            this, SLOT(slotDockBarContextMenuRequested(QPoint)));
 
     rightBarWidget = new IdealButtonBarWidget(Qt::RightDockWidgetArea, this);
     rightBarWidget->hide();
+    connect(rightBarWidget, SIGNAL(customContextMenuRequested(QPoint)),
+            this, SLOT(slotDockBarContextMenuRequested(QPoint)));
 
     bottomBarWidget = new IdealButtonBarWidget(Qt::BottomDockWidgetArea, this);
     bottomStatusBarLocation = bottomBarWidget->corner();
+    connect(bottomBarWidget, SIGNAL(customContextMenuRequested(QPoint)),
+            this, SLOT(slotDockBarContextMenuRequested(QPoint)));
 
     topBarWidget = new IdealButtonBarWidget(Qt::TopDockWidgetArea, this);
     topBarWidget->hide();
+    connect(topBarWidget, SIGNAL(customContextMenuRequested(QPoint)),
+            this, SLOT(slotDockBarContextMenuRequested(QPoint)));
 
     m_mainLayout = new IdealMainLayout(this);
     m_mainLayout->addButtonBar(leftBarWidget, IdealMainLayout::Left);
@@ -813,6 +865,14 @@ void IdealMainWidget::showDockWidget(IdealDockWidget * dock, bool show)
         focusEditor();
 }
 
+void IdealMainWidget::slotDockBarContextMenuRequested(const QPoint& position)
+{
+    IdealButtonBarWidget* bar = qobject_cast<IdealButtonBarWidget*>(sender());
+    Q_ASSERT(bar);
+
+    emit dockBarContextMenuRequested(bar->area(), bar->mapToGlobal(position));
+}
+
 IdealSplitterHandle::IdealSplitterHandle(Qt::Orientation orientation, QWidget* parent, IdealMainLayout::Role resizeRole)
     : QWidget(parent)
     , m_orientation(orientation)
@@ -831,7 +891,7 @@ void IdealSplitterHandle::paintEvent(QPaintEvent *)
     options.initFrom(this);
 
     if (m_orientation == Qt::Vertical)
-	options.state |= QStyle::State_Horizontal;
+    options.state |= QStyle::State_Horizontal;
 
     options.state |= QStyle::State_Enabled;
 
@@ -1028,6 +1088,11 @@ void IdealMainWidget::selectPreviousDock()
 
     IdealButtonBarWidget* bar = barForRole(role);
 
+    if ( bar->actions().isEmpty() ) {
+        // happens when all docks in a bar get removed
+        return;
+    }
+
     int index = bar->actions().indexOf(m_dockwidget_to_action.value(dock));
 
     if (index < 1)
@@ -1069,79 +1134,4 @@ QWidget *IdealMainWidget::statusBarLocation() const
     return bottomStatusBarLocation;
 }
 
-IdealDockWidgetButton::IdealDockWidgetButton(QWidget *parent)
-    : QToolButton(parent)
-{
-    setFocusPolicy(Qt::NoFocus);
-}
-
-IdealDockWidgetButton::~IdealDockWidgetButton()
-{
-}
-
-QSize IdealDockWidgetButton::sizeHint() const
-{
-    ensurePolished();
-
-    int size = 0;
-
-    if (! icon().isNull()) {
-	const QPixmap pix =
-	    icon().pixmap(style()->pixelMetric(QStyle::PM_SmallIconSize));
-
-	size += qMax(pix.width(), pix.height());
-    }
-
-    const int titleBarButtonMargin =
-	style()->pixelMetric(QStyle::PM_DockWidgetTitleBarButtonMargin);
-
-    size += titleBarButtonMargin * 2;
-
-    return QSize(size, size);
-}
-
-QSize IdealDockWidgetButton::minimumSizeHint() const
-{ return sizeHint(); }
-
-void IdealDockWidgetButton::enterEvent(QEvent *event)
-{
-    if (isEnabled())
-	update();
-
-    QToolButton::enterEvent(event);
-}
-
-void IdealDockWidgetButton::leaveEvent(QEvent *event)
-{
-    if (isEnabled())
-	update();
-
-    QToolButton::leaveEvent(event);
-}
-
-void IdealDockWidgetButton::paintEvent(QPaintEvent *)
-{
-    QStylePainter painter(this);
-
-    QStyleOptionToolButton options;
-    options.init(this);
-    options.state |= QStyle::State_AutoRaise;
-
-    if (isEnabled() && underMouse() && ! isChecked() && ! isDown())
-	options.state |= QStyle::State_Raised;
-    if (isChecked())
-	options.state |= QStyle::State_On;
-    if (isDown())
-	options.state |= QStyle::State_Sunken;
-
-    options.subControls = QStyle::SC_None;
-    options.activeSubControls = QStyle::SC_None;
-    options.icon = icon();
-    options.arrowType = Qt::NoArrow;
-    options.features = QStyleOptionToolButton::None;
-    const int size = style()->pixelMetric(QStyle::PM_SmallIconSize);
-    options.iconSize = QSize(size, size);
-    painter.drawComplexControl(QStyle::CC_ToolButton, options);
-}
-
 #include "ideal.moc"
