@@ -36,6 +36,7 @@
 class KAction;
 class KActionMenu;
 class KActionCollection;
+class KMenu;
 
 namespace Sublime {
 
@@ -67,24 +68,6 @@ private:
     bool showingIndicator;
 };
 
-class IdealDockWidgetButton: public QToolButton
-{
-    Q_OBJECT
-
-public:
-    IdealDockWidgetButton(QWidget *parent = 0);
-    virtual ~IdealDockWidgetButton();
-
-    QSize sizeHint() const;
-    QSize minimumSizeHint() const;
-
-protected:
-    virtual void enterEvent(QEvent *event);
-    virtual void leaveEvent(QEvent *event);
-    virtual void paintEvent(QPaintEvent *event);
-};
-
-
 class IdealButtonBarWidget: public QWidget
 {
     Q_OBJECT
@@ -101,6 +84,8 @@ public:
 
     Qt::Orientation orientation() const;
 
+    Qt::DockWidgetArea area() const;
+
     IdealDockWidget* widgetForAction(QAction* action) const;
 
     QWidget* corner();
@@ -118,7 +103,6 @@ private:
     Qt::DockWidgetArea _area;
     QHash<QAction *, IdealToolButton *> _buttons;
     QHash<QAction *, IdealDockWidget*> _widgets;
-    QActionGroup* _actions;
     QWidget *_corner;
 };
 
@@ -147,8 +131,11 @@ public:
 
     virtual bool event(QEvent *event);
 
+public slots:
+    /// The IdealToolButton also connects to this slot to show the same context menu.
+    void contextMenuRequested(const QPoint &point);
+
 protected: // QWidget overrides
-    virtual void contextMenuEvent(QContextMenuEvent *);
     virtual void mouseDoubleClickEvent(QMouseEvent *event);
 
 Q_SIGNALS:
@@ -220,6 +207,10 @@ public:
 Q_SIGNALS:
     void dockShown(Sublime::View*, Sublime::Position pos, bool shown);
 
+    /// Emitted, when a context menu is requested on one of the dock bars.
+    /// When no actions gets associated to the KMenu, it won't be shown.
+    void dockBarContextMenuRequested(Qt::DockWidgetArea area, const QPoint& position);
+
 public Q_SLOTS:
     void showLeftDock(bool show);
     void showRightDock(bool show);
@@ -233,6 +224,11 @@ public Q_SLOTS:
     void selectPreviousDock();
     void removeView();
 
+private Q_SLOTS:
+    /// Map context menu event from dockbar to area and propagate to main window.
+    /// @see dockBarContextMenuRequested()
+    void slotDockBarContextMenuRequested(const QPoint& position);
+
 private:
     // helpers for toggleDocksShown
     bool allDocksHidden();
