@@ -358,7 +358,7 @@ QSize IdealMainLayout::minimumSize() const
             minWidth += qMax(softMinWidth, itemSizeHint.width() + splitterWidth());
         }
 
-        m_min = QSize(minHeight, minWidth);
+        m_min = QSize(minWidth, minHeight);
         m_minDirty = true;
     }
 
@@ -736,6 +736,9 @@ void IdealMainLayout::removeWidgets(Role role)
 
 void IdealMainLayout::removeWidget(QWidget * widget, Role role)
 {
+    if (m_maximizedWidget)
+        maximizeWidget(0);
+
     DockArea* area = m_items[role];
     area->removeWidget(widget);
     if (area->items().isEmpty())
