@@ -81,7 +81,7 @@ MainWindowPrivate::MainWindowPrivate(MainWindow *w, Controller* controller)
     action = new KAction(i18n("Focus Editor"), this);
     action->setShortcuts(QList<QKeySequence>() << (Qt::META | Qt::CTRL | Qt::Key_E) << Qt::META + Qt::Key_C);
     connect(action, SIGNAL(triggered(bool)), this, SLOT(focusEditor()));
-    m_mainWindow->actionCollection()->addAction("focus_editor", action);
+    ac->addAction("focus_editor", action);
 
     action = new KAction(i18n("Hide/Restore Docks"), this);
     action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_H);
@@ -229,6 +229,8 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
             //we need to create view container
             container = new Container(splitter);
             connect(container, SIGNAL(activateView(Sublime::View*)), d->m_mainWindow, SLOT(activateView(Sublime::View*)));
+            connect(container, SIGNAL(tabContextMenuRequested(Sublime::View*,KMenu*)),
+                    d->m_mainWindow, SLOT(tabContextMenuRequested(Sublime::View*,KMenu*)));
             connect(container, SIGNAL(closeRequest(QWidget*)),
                     d, SLOT(widgetCloseRequest(QWidget*)));
             splitter->addWidget(container);
@@ -342,6 +344,9 @@ void MainWindowPrivate::recreateCentralWidget()
             SIGNAL(widgetResized(IdealMainLayout::Role, int)),
             this,
             SLOT(widgetResized(IdealMainLayout::Role, int)));
+
+    connect(idealMainWidget, SIGNAL(dockBarContextMenuRequested(Qt::DockWidgetArea, const QPoint&)),
+            m_mainWindow, SLOT(dockBarContextMenuRequested(Qt::DockWidgetArea, const QPoint&)));
 }
 
 void MainWindowPrivate::
@@ -507,7 +512,7 @@ void MainWindowPrivate::aboutToRemoveView(Sublime::AreaIndex *index, Sublime::Vi
     m_mainWindow->setActiveView(0L);
 }
 
-void MainWindowPrivate::toolViewAdded(Sublime::View */*toolView*/, Sublime::Position position)
+void MainWindowPrivate::toolViewAdded(Sublime::View* /*toolView*/, Sublime::Position position)
 {
     IdealToolViewCreator toolViewCreator(this);
     area->walkToolViews(toolViewCreator, position);
@@ -585,12 +590,9 @@ void MainWindowPrivate::widgetCloseRequest(QWidget* widget)
 }
 
 void MainWindowPrivate::toggleArea ( int index ) {
-#if QT_VERSION >= 0x040500
     m_mainWindow->controller()->showArea(areaSwitcher->tabBar->areaId(index), m_mainWindow);
-#endif
 }
 
-#if QT_VERSION >= 0x040500
 void AreaTabBar::paintEvent ( QPaintEvent* ev ) {
     QTabBar::paintEvent(ev);
     if ( currentIndex() != -1 ) {
@@ -728,10 +730,13 @@ AreaTabBar::AreaTabBar ( QWidget* parent ) : QTabBar ( parent ), m_currentIndex
     setExpanding ( false );
     setLayoutDirection ( Qt::RightToLeft );
     setDrawBase ( false );
+    setFocusPolicy( Qt::NoFocus );
     QPalette pal = palette();
 }
 
-AreaTabButton::AreaTabButton ( QString text, QIcon icon, uint iconSize, QWidget* parent, bool isCurrent, QWidget* _customButtonWidget ) : QWidget ( parent ), m_isCurrent ( isCurrent ), customButtonWidget(_customButtonWidget) {
+AreaTabButton::AreaTabButton ( QString text, QIcon icon, uint iconSize, QWidget* parent, bool isCurrent, QWidget* _customButtonWidget )
+	: QWidget ( parent ), customButtonWidget(_customButtonWidget), m_isCurrent ( isCurrent )
+{
     QHBoxLayout* layout = new QHBoxLayout ( this );
     iconLabel = new QLabel ( this );
     iconLabel->setPixmap ( icon.pixmap ( QSize ( iconSize, iconSize ) ) );
@@ -747,7 +752,6 @@ AreaTabButton::AreaTabButton ( QString text, QIcon icon, uint iconSize, QWidget*
     layout->addWidget ( iconLabel );
     layout->setMargin ( 0 );
 }
-#endif
 
 void MainWindowPrivate::setTabBarLeftCornerWidget(QWidget* widget)
 {
