@@ -37,6 +37,7 @@ public:
     Document *doc;
     QWidget *widget;
     void unsetWidget();
+    View::WidgetOwnership ws;
 
 private:
     View *view;
@@ -52,16 +53,20 @@ void ViewPrivate::unsetWidget()
     widget = 0;
 }
 
-View::View(Document *doc)
+View::View(Document *doc, WidgetOwnership ws )
     :QObject(doc), d(new ViewPrivate(this))
 {
     d->doc = doc;
+    d->ws = ws;
 }
 
 View::~View()
 {
-    if (d->widget)
-        delete d->widget;
+    if (d->widget && d->ws == View::TakeOwnership ) {
+        d->widget->hide();
+        d->widget->setParent(0);
+        d->widget->deleteLater();
+    }
     delete d;
 }
 
@@ -125,6 +130,26 @@ void View::notifyPositionChanged(int newPositionInArea)
     emit positionChanged(this, newPositionInArea);
 }
 
+
+ToolView::ToolView(Document* doc, View::WidgetOwnership ws)
+    : View( doc, ws )
+{
+}
+
+
+QWidget* ToolView::createWidget(QWidget* parent)
+{
+    QWidget* w = Sublime::View::createWidget(parent);
+    ToolDocument* tooldoc = dynamic_cast<ToolDocument*>( document() );
+    if( tooldoc->factory()->viewsWantProgressIndicator() ) 
+    {
+        connect( w, SIGNAL( showProgressIndicator() ), SIGNAL( showProgressIndicator() ) );
+        connect( w, SIGNAL( hideProgressIndicator() ), SIGNAL( hideProgressIndicator() ) );
+    }
+    return w;
+}
+
+
 }
 
 #include "view.moc"
