@@ -37,11 +37,15 @@ public:
     QString m_defaultGroup;
 };
 
+static const QString defaultEnvGroupKey = "Default Environment Group";
+static const QString envGroup = "Environment Settings";
+static const QString groupListKey = "Group List";
 
 void decode( KConfigGroup cfg, EnvironmentGroupListPrivate* d )
 {
-    d->m_defaultGroup = cfg.readEntry( "Default EnvironmentGroup", QString( "default" ) );
-    foreach( const QString &envgrpname, cfg.groupList() )
+    d->m_defaultGroup = cfg.readEntry( defaultEnvGroupKey, QString( "default" ) );
+    QStringList grouplist = cfg.readEntry( groupListKey, QStringList() << "default" );
+    foreach( const QString &envgrpname, grouplist )
     {
         KConfigGroup envgrp( &cfg, envgrpname );
         QMap<QString,QString> variables;
@@ -51,17 +55,19 @@ void decode( KConfigGroup cfg, EnvironmentGroupListPrivate* d )
         }
         d->m_groups.insert( envgrpname, variables );
     }
-
-    // If the defaultgroup doesn't exist yet create it
-    if( !d->m_groups.contains( d->m_defaultGroup ) )
-    {
-        d->m_groups.insert( d->m_defaultGroup, QMap<QString,QString>() );
-    }
 }
 
 void encode( KConfigGroup cfg, EnvironmentGroupListPrivate* d )
 {
-    cfg.writeEntry( "Default Environment Group", d->m_defaultGroup );
+    cfg.writeEntry( defaultEnvGroupKey, d->m_defaultGroup );
+    cfg.writeEntry( groupListKey, d->m_groups.keys() );
+    foreach( const QString &group, cfg.groupList() ) 
+    {
+        if( !d->m_groups.keys().contains( group ) ) 
+        {
+            cfg.deleteGroup( group );
+        }
+    }
     foreach( const QString &group, d->m_groups.keys() )
     {
         KConfigGroup envgrp( &cfg, group );
@@ -76,14 +82,14 @@ void encode( KConfigGroup cfg, EnvironmentGroupListPrivate* d )
 EnvironmentGroupList::EnvironmentGroupList( KSharedConfigPtr config )
     : d(new EnvironmentGroupListPrivate)
 {
-    KConfigGroup cfg( config, "Environment Settings" );
+    KConfigGroup cfg( config, envGroup );
     decode( cfg, d );
 }
 
 EnvironmentGroupList::EnvironmentGroupList( KConfig* config )
     : d(new EnvironmentGroupListPrivate)
 {
-    KConfigGroup cfg( config, "Environment Settings" );
+    KConfigGroup cfg( config, envGroup );
     decode( cfg, d );
 }
 
@@ -119,14 +125,15 @@ void EnvironmentGroupList::setDefaultGroup( const QString& group )
 
 void EnvironmentGroupList::saveSettings( KConfig* config ) const
 {
-    KConfigGroup cfg(config, "Environment Settings" );
+    KConfigGroup cfg(config, envGroup );
     encode( cfg, d );
+    config->sync();
 }
 
 void EnvironmentGroupList::loadSettings( KConfig* config )
 {
     d->m_groups.clear();
-    KConfigGroup cfg(config, "Environment Settings" );
+    KConfigGroup cfg(config, envGroup );
     decode( cfg, d );
 }
 
@@ -145,10 +152,10 @@ EnvironmentGroupList::EnvironmentGroupList()
 {
 }
 
-QStringList EnvironmentGroupList::createEnvironment(const QString & group, const QStringList & defaults) const
+QStringList EnvironmentGroupList::createEnvironment( const QString & group, const QStringList & defaultEnvironment ) const
 {
     QMap<QString, QString> retMap;
-    foreach( const QString &line, defaults )
+    foreach( const QString &line, defaultEnvironment )
     {
         QString varName = line.section( '=', 0, 0 );
         QString varValue = line.section( '=', 1 );
