@@ -60,9 +60,15 @@
 
 #include <config-kdevplatform.h>
 #if HAVE_KOMPARE
-#include <KTemporaryFile>
 #include <interfaces/ipatchdocument.h>
 #endif
+#include <interfaces/ipatchsource.h>
+#include <KTemporaryFile>
+#include <qtextedit.h>
+#include "vcsstatusinfo.h"
+#include <qboxlayout.h>
+#include <qlabel.h>
+#include "vcsdiffpatchsources.h"
 
 namespace KDevelop
 {
@@ -230,46 +236,6 @@ void VcsPluginHelper::diffToHead()
     d->plugin->core()->runController()->registerJob(job);
 }
 
-#if HAVE_KOMPARE
-void showDiff(const KDevelop::VcsDiff& d)
-{
-    ICore::self()->uiController()->switchToArea("test", KDevelop::IUiController::ThisWindow);
-    foreach(const VcsLocation& l, d.leftTexts().keys())
-    {
-        KUrl to;
-        if(d.rightTexts().contains(l))
-        {
-            KTemporaryFile temp2;
-            temp2.setSuffix("2.patch");
-            temp2.setAutoRemove(false);
-            temp2.open();
-            QTextStream t2(&temp2);
-            t2 << d.rightTexts()[l];
-            temp2.close();
-            to=temp2.fileName();
-        }
-        else
-            to=l.localUrl();
-        
-        KUrl fakeUrl(to);
-        fakeUrl.setScheme("kdevpatch");
-        
-        IDocumentFactory* docf=ICore::self()->documentController()->factory("text/x-patch");
-        IDocument* doc=docf->create(fakeUrl, ICore::self());
-        IPatchDocument* pdoc=dynamic_cast<IPatchDocument*>(doc);
-        
-        Q_ASSERT(pdoc);
-        ICore::self()->documentController()->openDocument(doc);
-        pdoc->setDiff(d.leftTexts()[l], to);
-    }
-}
-#else
-void showDiff(const KDevelop::VcsDiff& d)
-{
-    ICore::self()->documentController()->openDocumentFromText(d.diff());
-}
-#endif
-
 void VcsPluginHelper::diffJobFinished(KJob* job)
 {
     KDevelop::VcsJob* vcsjob = dynamic_cast<KDevelop::VcsJob*>(job);
@@ -278,7 +244,7 @@ void VcsPluginHelper::diffJobFinished(KJob* job)
     if (vcsjob) {
         if (vcsjob->status() == KDevelop::VcsJob::JobSucceeded) {
             KDevelop::VcsDiff d = vcsjob->fetchResults().value<KDevelop::VcsDiff>();
-            showDiff(d);
+            showVcsDiff(new VCSDiffPatchSource(repairDiff(d.diff())));
         } else {
             KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), vcsjob->errorString(), i18n("Unable to get difference."));
         }
@@ -358,13 +324,12 @@ void VcsPluginHelper::commit()
 {
     Q_ASSERT(!d->ctxUrls.isEmpty());
     KDevelop::VcsCommitDialog* dlg = new KDevelop::VcsCommitDialog(d->plugin, d->plugin->core()->uiController()->activeMainWindow());
-    dlg->setCommitCandidates(d->ctxUrls);
 //     KConfigGroup vcsGroup(KSharedConfig::openConfig(componentData()), "VcsCommon");
 //     dlg->setOldMessages(vcsGroup.readEntry("OldCommitMessages", QStringList()));
 //     dlg->setRecursive(true);
     connect(dlg, SIGNAL(doCommit(KDevelop::VcsCommitDialog*)), this, SLOT(executeCommit(KDevelop::VcsCommitDialog*)));
     connect(dlg, SIGNAL(cancelCommit(KDevelop::VcsCommitDialog*)), this, SLOT(cancelCommit(KDevelop::VcsCommitDialog*)));
-    dlg->show();
+    dlg->setCommitCandidatesAndShow(d->ctxUrls);
 }
 
 void VcsPluginHelper::executeCommit(KDevelop::VcsCommitDialog* dlg)
@@ -375,7 +340,7 @@ void VcsPluginHelper::executeCommit(KDevelop::VcsCommitDialog* dlg)
     vcsGroup.writeEntry("OldCommitMessages", oldMessages);
 
     KDevelop::IBasicVersionControl* iface = dlg->versionControlPlugin()->extension<KDevelop::IBasicVersionControl>();
-    d->plugin->core()->runController()->registerJob(iface->commit(dlg->message(), dlg->checkedUrls(),
+    d->plugin->core()->runController()->registerJob(iface->commit(dlg->message(), dlg->determineUrlsForCheckin(),
             dlg->recursive() ?  KDevelop::IBasicVersionControl::Recursive : KDevelop::IBasicVersionControl::NonRecursive));
 
     dlg->deleteLater();
